<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AlbumsController extends AppController {
	public $scaffold;
	public $uses = array('Album', 'Search');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $components = array('PhpThumb');
	public $paginate = array(
		'limit' => 42,
		'page' => 1,
		'conditions' => array(),
		'fields' => array('Album.id', 'Album.title', 'Album.series_id', 'Series.name'),
		'order'=>array('Series.name' => 'asc', 'Album.cycle' => 'asc', 'Album.number' => 'asc', 'Album.number_suffix' => 'asc'),
		'recursive' => 1
	);
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);

	function index() {
		$this->paginate['limit'] = ($perColumn = Configure::read('Demyo.itemsPerIndexColumns')) * Configure::read('Demyo.columnsPerIndexPage');
		$this->set('perColumn', $perColumn);
		$relationsToKeep = array('belongsTo' => array('Series'));
		$albums = $this->performIndexSearch($relationsToKeep);
		$this->set('albums', $albums);
		$this->render('index'); // Forced when coming from a search
	}
	
	function covers() {
		$this->helpers[] = 'AlbumTools';
		$this->helpers[] = 'Colorbox';
		$this->paginate['limit'] = Configure::read('Demyo.Gallery_thumbsPerPage');
		$this->paginate['fields'] = array('Album.id', 'Album.cycle', 'Album.number', 'Album.number_suffix', 'Album.title', 'Album.series_id', 'Album.cover_id', 'Series.name', 'Cover.url', 'Cover.is_local');

		$relationsToKeep = array('belongsTo' => array('Series', 'Cover'));
		$albums = $this->performIndexSearch($relationsToKeep);

		foreach ($albums as &$album)
			if (!empty($album['Cover']['url']))
				$album['Cover']['thumb_url'] = $this->PhpThumb->getThumbnail($album['Cover']['url']);

		$this->set('albums', $albums);
		$this->render('covers'); // Forced when coming from a search
	}
	
	private function performIndexSearch($relationsToKeep) {
		$this->helpers[] = 'MyPaginator';
		$this->helpers[] = 'Text';
		
		$filters = $this->getFilters(array('withBinding', 'fromPublisher', 'fromCollection', 'onlyWithCover', 'searchId'));
		extract($filters);
		
		$scope = $this->paginate['conditions']; // Start from conditions possibly set by the search.
				// Especially useful to make the "startsWith" parameter work in searches.
		if ($withBinding !== null) {
			array_push($scope, array('Album.binding_id' => $withBinding));
			$this->set('title_for_layout', __s('Viewing albums of binding \'%s\'', $this->Album->Binding->getName($withBinding)));
		}
		if ($fromPublisher !== null) {
			array_push($scope, array('Album.publisher_id' => $fromPublisher));
			$this->set('title_for_layout', __s('Viewing albums from %s', $this->Album->Publisher->getName($fromPublisher)));
		}
		if ($fromCollection !== null) {
			array_push($scope, array('Album.collection_id' => $fromCollection));
			$this->set('title_for_layout', __s('Viewing albums from %s', $this->Album->Collection->getName($fromCollection)));
		}
		if ($onlyWithCover === 'true') {
			array_push($scope, array('Album.cover_id IS NOT NULL'));
		}
		$this->set('onlyWithCover', $onlyWithCover === 'true');
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Album->findPageNumberStartsWith('Series.name', $this->params['named']['startsWith'], $this->paginate['limit'], $scope);
		}
		
		$this->params['pass'] = array_filter($filters);
		$this->Album->unbindAll($relationsToKeep, false);
		return $this->paginate('Album', $scope);
	}

	function view($id = null) {
		$this->helpers[] = 'AlbumTools';
		$this->helpers[] = 'Chart';
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'List';
		$this->helpers[] = 'Prefetch';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'Rte';
		$this->helpers[] = 'TagTools';
		
		if ($id == null)
			$this->redirect('/albums');

		$this->Album->id = $id;
		$this->Album->contain(array(
			'Publisher.id', 'Publisher.name',
			'Collection.id', 'Collection.name',
			'Binding',
			'Series.id', 'Series.name',
			'Artist.id', 'Artist.name', 'Artist.fname', 'Artist.nickname',
			'Writer.id', 'Writer.name', 'Writer.fname', 'Writer.nickname',
			'Colorist.id', 'Colorist.name', 'Colorist.fname', 'Colorist.nickname',
			'Tag',
			'Cover',
			'Image',
			'Borrower',
			'Derivative.id',
			'Derivative.Image',
			'AlbumPrice'
		));
		if (! $album = $this->Album->read())
			$this->errorNoSuchEntry(__('This album does not exist.', true));
		
		// Fake album data as if the images from the derivatives were actually
		// from the album
		foreach ($album['Derivative'] as &$deriv)
			$album['Image'] = array_merge($album['Image'], $deriv['Image']);
		unset($album['Derivative']);
		
		// Generate thumbnails
		if (! empty($album['Cover']['id']))
			$album['Cover']['thumb_url'] = $this->PhpThumb->getThumbnail($album['Cover']['url']);
		foreach($album['Image'] as &$image)
			$image['thumb_url'] = $this->PhpThumb->getThumbnail($image['url']);
		
		$album['Album']['derivative_count'] = $this->Album->Derivative->getDerivativesCount('Album', $id);
		
		$this->set('album', $album);
		$this->set('surroundings', $this->Album->Series->getSurroundings($album['Album']['series_id'], $album['Album']['id']));
	}
	
	function add() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'Rte';
		
		if (!empty($this->data)) {
			$series_id = empty($this->data['Album']['series_id'])?0:$this->data['Album']['series_id'];
			$this->saveAlbum(array(
				'message' => __('The album has been added', true),
				'gotos' => array(
						'/albums/view/$_MODEL_ID_$' => __('View album', true),
						'/series/view/'.$series_id => __('View series', true))
				), false);
		}
		
		else {
			// Hack POST array to select default values, some of them being
			// guessed from the most recent album (they rarely change)
			
			if (isset($this->params['named']['series'])) {
				// Series			
				$this->data['Album']['series_id'] = $this->params['named']['series'];

				$this->Album->contain(array(
					'Writer.id', 'Artist.id', 'Colorist.id'
				));
				$lastAlbum = $this->Album->find('first', array(
					'conditions' => array('series_id' =>  $this->params['named']['series']),
					'order' => array('cycle' => 'DESC', 'number' => 'DESC', 'number_suffix' => 'DESC'),
				));
				
				if (! empty($lastAlbum)) {
					// Publisher
					$this->data['Album']['publisher_id'] = $lastAlbum['Album']['publisher_id'];
					
					// Collection
					$this->data['Album']['collection_id'] = $lastAlbum['Album']['collection_id'];
					
					// Binding
					$this->data['Album']['binding_id'] = $lastAlbum['Album']['binding_id'];
					
					// Dimensions
					if (! empty($lastAlbum['Album']['width']))
						$this->data['Album']['width'] = $lastAlbum['Album']['width'];
					if (! empty($lastAlbum['Album']['height']))
						$this->data['Album']['height'] = $lastAlbum['Album']['height'];
					
					// Authors
					$this->data['Colorist'] = $lastAlbum['Colorist'];
					$this->data['Artist'] = $lastAlbum['Artist'];
					$this->data['Writer'] = $lastAlbum['Writer'];
				}
			}
			
			
			$this->generateLists();
		}
	}
	
	function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'Rte';
		
		if (empty($this->data)) {
			$this->Album->id = $id;
			$this->Album->contain(array('Writer.id', 'Artist.id', 'Colorist.id', 'Tag.name', 'Image.id', 'AlbumPrice'));
			if (! $this->data = $this->Album->read())
				$this->errorNoSuchEntry(__('This album does not exist.', true));
			
			// Fix dates
			foreach (array('first_edition', 'this_edition', 'purchase_date') as $date)
				if (isset($this->data['Album'][$date]) && $this->data['Album'][$date] === '0000-00-00')
					$this->data['Album'][$date] = '';

			// Fix tags: fetch names, format as CSV, remove old info. Name-to-id conversion is done when saving
			$tags = array();
			foreach($this->data['Tag'] as $tag)
				array_push($tags, $tag['name']);
			unset($this->data['Tag']);
			$this->data['Tag']['Tag'] = implode(',', $tags);
			
			$this->generateLists();
		}
		
		else {
			debug('save');
			$this->saveAlbum(array(
				'message' => __('The album has been updated', true),
				'gotos' => array(
					'/albums/view/'.$this->data['Album']['id'] => __('View album', true),
					'/series/view/'.$this->data['Album']['series_id'] => __('View series', true))
				));
		}
	}
	
	/** Saves (add or edit) an album using $this->data */
	private function saveAlbum($flashOptions, $redirectToPrevious = true) {
		// Recover tag IDs, as we previously gave names
		$this->data['Tag']['Tag'] = $this->Album->Tag->getFromCSV($this->data['Tag']['Tag']);
		
		// Clear purchase date for wishlist items
		if (! empty($this->data['Album']['wishlist']))
			$this->data['Album']['purchase_date'] = '';
		
		// Check prices
		if (! empty($this->data['AlbumPrice']))
			foreach($this->data['AlbumPrice'] as $key => $value) {
				if (empty($value['date']) || empty($value['price']))
					unset($this->data['AlbumPrice'][$key]);
			}
		
		$this->Album->set($this->data);
		if ($this->Album->validates() === true) {
			// Unfortunately, MySQL doesn't support nested transactions so if
			// the saveAll call fails, we lose the price history. However, since
			// a validation check is done before deleting, this should only
			// happen in case of an inconsistent database.
			if (! empty($this->data['Album']['id'])) // Obviously, this step is useless for new albums
				$this->Album->AlbumPrice->deleteForAlbum($this->data['Album']['id']);
			
			if ($this->Album->saveAll($this->data, array('validate' => false))) {
				extract($flashOptions);
				$this->smartFlash($message, 'info', $gotos, $redirectToPrevious);
			} else
				$this->generateLists(); 
		} else
			$this->generateLists();
	}
	
	public function purchases($type = 'global', $period = false) {
		$this->layout = 'simple';
		$this->helpers[] = 'Chart';
		
		if ($type === 'global') {
			$this->set('stats', $this->Album->getPurchaseStatisticsByYear());
			$this->set('type', 'global');
			$this->set('limit', Configure::read('Demyo.Stats_years'));
		} else if ($type === 'monthly') {
			$this->set('stats', $this->Album->getPurchaseStatisticsForYear($period));
			$this->set('type', 'monthly');
			$this->set('period', $period);
			$this->set('limit', 12);
		} else
			return $this->errorNoSuchEntry(__('No such statistics', true));
	}
	
	private function generateLists() {
		$this->loadModel('Author'); // Linked as Artist, Writer and Colorist, but not Author
		
		$this->set('seriesList', $this->Album->Series->find('list'));
		$this->set('publishersList', $this->Album->Publisher->find('list'));
		$this->set('collectionsList', $this->Album->Collection->find('list'));
		$this->set('bindingsList', $this->Album->Binding->find('list'));
		$this->set('authorsList', $this->Author->find('list'));
		$this->set('tagsList', $this->Album->Tag->find('list'));
		$this->set('imagesList', $this->Album->Image->find('list'));
	}
	
	function fromSeries($id = null) {
		$this->layout = 'ajax';
		
		if ($id != null)
			$this->set('albums', $this->Album->find('list', array('conditions' => 'Album.series_id='.$id, 'order' => 'cycle ASC, number ASC, number_suffix ASC')));
		else
			$this->set('albums', array());
	}
	
	public function search() {
		$this->Search->id = $this->params['named']['searchId'];
		if (! $search = $this->Search->read())
			$this->errorNoSuchEntry(__('This search does not exist.', true));
		$this->Search->refreshUsage($this->Search->id);

		$this->set('search_is_permanent', $search['Search']['status'] == Search::STATUS_PERMANENT);
		
		// Change conditions on-the-fly
		$this->paginate['conditions'] = $this->Search->decodeCriteria($search['Search']['criteria']);

		// Choose rendering
		$parameters = $this->Search->decodeParameters($search['Search']['parameters']);
		if (isset($parameters['view_type']) && $parameters['view_type'] == 'covers') {
			$this->covers();
		} else {
			$this->index();
		}
	}
}

?>
