<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AppController extends Controller {
	public $persistModel = true;
	public $helpers = array('Asset.asset', 'Session');
	
	/** List of actions that will trigger prepareSmartFlash().
	 * Defaults to add, edit, delete.
	 *
	 * @var array
	 * @see prepareSmartFlash()
	 */
	protected $smartFlashActions = array('add', 'edit');
	/** List of action that imply a write access and should be protected by
	 * security checks. Each action name can be associated to an array of
	 * of options:
	 *		. onlyPost (boolean): If the action should only be considered if
	 *	$this->data or $this->params['form'] is not empty.
	 * @var array
	 */
	protected $writeActions = array('add' => array('onlyPost' => true), 'edit' => array('onlyPost' => true), 'save', 'delete');
	
	protected function error($title, $message, $skipSanitize = true) {
		$this->cakeError('error', array(array('message' => $message, 'base' => '', 'code' => '', 'name' => $title, 'skipSanitize' => $skipSanitize)));
		return false;
	}

	public function delete($id) {
		$modelName = Inflector::singularize($this->name);
		$redirect = '/'.Inflector::tableize($modelName);
		$this->$modelName->id = $id;
		if (!$this->$modelName->exists() || $this->$modelName->delete($id)) {
			$this->smartFlash(__('The item was sucessfully deleted.', true), 'info', array($redirect => ''), false);
		} else {
			$redirect = $this->referer($redirect, true);
			$this->smartFlash($this->$modelName->errorDetails, 'error', array($redirect => ''), false);
		}
	}
	
	protected function errorNoSuchEntry($message) {
		header("HTTP/1.0 404 Not Found");
		$this->cakeError('error',
			array('code' => '',
				'name' => __('Not found', true),
				'message' => $message
			)
		);
	}
	
	/** Returns a list of filter => value, based on the names from the first
	 * parameter and fetched either from URL parameters or POST data.
	 * This method has no effect on the object state.
	 *
	 * @param array $names the accepted parameters (other URL parameters are
	 * ignored)
	 * @return array An array of parameter => value, with missing or empty
	 * parameters marked as null.
	 */
	protected function getFilters($names) {
		$values = array();
		
		foreach ($names as $parameter) {
			$val = null;
			
			if (isset($this->params['named'][$parameter]))
				$val = $this->params['named'][$parameter];
			else if (isset($this->data[$parameter]))
				$val = $this->data[$parameter];
			
			if (empty($val) || $val === '0')
				$val = null;
			
			$values[$parameter] = $val;
		}
		
		return $values;
	}
	
	/** Prepares pagination on a custom set of data (i.e. not returned by
	 * paginate()).
	 *
	 * @param integer $page The current page number
	 * @param integer $limit The number of items to show per page
	 * @param integer $totalItems The total number of items in the non-paginated
	 * set
	 * @param array $params An optionnal array of URL parameters to pass from
	 * one page to the other
	 */
	protected function prepareManualPagination($model, $page, $limit, $totalItems, $params = array()) {
		$this->set('doPaginate', true);
		$this->params['paging'][$model]['page'] = $page;
		if (isset($this->params['named']['sort']))
			$params['sort'] = $this->params['named']['sort'];
		if (isset($this->params['named']['direction']))
			$params['direction'] = $this->params['named']['direction'];
		$this->params['paging'][$model]['pageCount'] = ceil($totalItems/$limit);
		$this->params['paging'][$model]['prevPage'] = $page > 1;
		$this->params['paging'][$model]['nextPage'] = $page < $this->params['paging'][$model]['pageCount'];
		$this->params['paging'][$model]['last'] = ceil($totalItems/$limit);
		$this->params['paging'][$model]['defaults'] = array('page' => 1, 'limit' => $limit);
		$this->params['paging'][$model]['options'] = array_merge($params, array('page' => $page, 'limit' => $limit));
	}
	
	/**
 	 * Called before the controller action. Overridden in subclasses.
 	 * Be sure to call parent::beforeFilter() when overriding this.
 	 */
	function beforeFilter() {
		$this->prepareSmartFlash();
		$this->checkWriteActions();
		
		// FS#308: disable caching when flashing a message
		if ($this->Session->check('Message.flash') || $this->Session->check('Message.smartFlash')) {
			Configure::write('Cache.check', false);
		}
		
		// Theming
		$theme = Configure::read('Demyo.theme');
		if (! empty($theme)) {
			$this->view = 'Theme';
			$this->theme = Configure::read('Demyo.theme');
		}
	}	
	
	/* Note to self: working with session isn't safe:
	 * - User browses series A
	 * - User edits album A.1 in a new tab (-> referer set to series B)
	 * - User browses series B
	 * - User edits album B.1 (-> referer set to series B)
	 * - User saves and is redirected to series B
	 * - User goes back to second tab, saves, and is redirected to series B
	 *   because the reference to A was overriden.
	 * 
	 * We cannot work with a stack of redirects either because there is no
	 * guarantee that they will be popped.
	 */
	/** Prepares a view variable for smart flash if the current action
	 * is present in the $smartFlashActions array.
	 *
	 * @param boolean $force true if the variable should be prepared
	 * even if the action isn't mentionned in $smartRedirectActions
	 */
	protected function prepareSmartFlash($force = false) {
		// Enable if forced or candidate, and if the referer is not the home page
		if (($force || in_array($this->params['action'], $this->smartFlashActions))
				&& isset($_SERVER['HTTP_REFERER'])
				&& $_SERVER['HTTP_REFERER'] !== Router::url('/', true)) {

			// Special case for validation issues (see bug #274)
			// If the previous page is the same as the current one, and the
			// smartFlash parameter was set there, recover it.
			// The strpos helps managing named parameters in url (add/toSeries:foo)
			if (!empty($this->params['form']['smartFlash'])
					&& strpos($_SERVER['HTTP_REFERER'], Router::url(null, true)) === 0) {
				$this->set('smartFlash', $this->params['form']['smartFlash']);
			} else {
				$this->set('smartFlash', $_SERVER['HTTP_REFERER']);
			}
		}

		else
			$this->set('smartFlash', '');
	}
	
	/** Redirects a user, showing him some information about the action that led to the redirect.
	 * Once redirected, the user may choose to visit another page from a list of recommendations.
	 * 
	 * @param $message The message to flash.
	 * @param $gotos An array of url => message pairs. URLs may contain the special
	 * string $_MODEL_ID_$, which will be substituted $this->ModelName->id.
	 * @param $pause An optional time to pause before redirecting (defaults to 1)
	 * @param $redirectToPrevious An optional parameter that sets whether the user
	 * should be redirected to the previous page automatically (true) or
	 * if it should be an option (false).
	 */
	protected function smartFlash($message, $type = 'info', $gotos = array(), $redirectToPrevious = true) {
		$modelName = Inflector::singularize($this->name);
		if (! empty($this->{$modelName}->id))
			$modelID = $this->{$modelName}->id;
		else
			$modelID = '';

		$redirectUrl = Router::url('/', true); // By default, redirect to home page

		if ($redirectToPrevious && !empty($this->params['form']['smartFlash'])) { // Redirect to page kept by smartFlash
			$redirectUrl = $this->params['form']['smartFlash'];
		}

		else if (sizeof($gotos) > 0) { // Redirect to first item in the list
			$url = key($gotos);
			$redirectUrl = Router::url(str_replace('$_MODEL_ID_$', $modelID, $url), true);
			// Item is removed from the list. If we redirect there, we don't want to see it twice.
			array_shift($gotos);
		}

		// Go through list of possible forwards, fixing potential URLs and removing
		// whatever links to the page we will be redirecting to
		$filteredGotos = array();
		foreach ($gotos as $url => $gotoMessage) {
			$url = Router::url(str_replace('$_MODEL_ID_$', $modelID, $url), true);
			if ($redirectUrl === $url)
				continue;

			$filteredGotos[] = array(
				'url' => $url,
				'message' => $gotoMessage
			);
		}

		if (!$redirectToPrevious && !empty($this->params['form']['smartFlash'])) { // Add "Back to previous" as an option
			$filteredGotos[] = array(
				'url' => $this->params['form']['smartFlash'],
				'message' => __('Back to previous page', true)
			);
		}

		$this->Session->setFlash($message, 'advancedFlash', array('gotos' => $filteredGotos, 'type' => $type), 'smartFlash');
		$this->redirect($redirectUrl);
	}

	private function checkWriteActions() {
		if (!in_array($this->params['action'], $this->writeActions)
				&& !isset($this->writeActions[$this->params['action']]))
			return;

		// Load options (with defaults)
		$options = array('onlyPost' => false);
		if (!empty($this->writeActions[$this->params['action']]))
			$options = array_merge($options, $this->writeActions[$this->params['action']]);

		if ($options['onlyPost'] && empty($this->data) && empty($this->params['form']))
			return;
		
		if (!isset($_SERVER['HTTP_REFERER'])
				|| strpos($_SERVER['HTTP_REFERER'], Router::url('/', true)) !== 0) {
			header("HTTP/1.0 403 Forbidden");
			$this->error(__('Unauthorized', true),
				__('Writing (editing, saving, deleting) to the database is only allowed from within Demyo. You cannot copy/paste such links, or follow them from a mail or forum.', true));
		}
	}
}
?>