<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AuthorsController extends AppController {
	public $scaffold;
	public $uses = array('Author');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $components = array('PhpThumb');
	public $paginate = array('limit' => 42, 'page' => 1, 'order'=>array('name' => 'asc'), 'fields' => array('id','name', 'fname'), 'recursive' => -1);
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH); 

	function index() {
		$this->helpers[] = 'MyPaginator';
		
		$this->paginate['limit'] = ($perColumn = Configure::read('Demyo.itemsPerIndexColumns')) * Configure::read('Demyo.columnsPerIndexPage');
		$this->set('perColumn', $perColumn);
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Author->findPageNumberStartsWith('name', $this->params['named']['startsWith'], $this->paginate['limit']);
		}
		
		$this->set('authors', $this->paginate('Author'));
	}
	
	function view($id = null) {
		$this->helpers[] = 'AlbumTools';
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'Rte';
		
		$this->Author->id = $id;
		$this->Author->contain(array('Image'));
		if (! $author = $this->Author->read())
			$this->errorNoSuchEntry(__('This author does not exist.', true));
		if (! empty($author['Image']['id']))
			$author['Image']['thumb_url'] = $this->PhpThumb->getThumbnail($author['Image']['url']);
		$this->loadModel('Derivative'); // Linked as Artist
		$author['Author']['derivative_count'] = $this->Derivative->getDerivativesCount('Artist', $id);
		$this->set('author', $author);
		
		
		foreach (array('Artist', 'Writer', 'Colorist') as $model) {
			$this->loadModel($model);
			$this->$model->contain(array(
				'Album.id', 'Album.cycle', 'Album.number', 'Album.number_suffix',
				'Album.title', 'Album.Series.name'
			));
			$result = $this->$model->find(array('id' => $id), array('id'));
			usort($result['Album'], array('Album', 'sortWithSeries'));
			$this->set("as$model", $result['Album']);
		}
    }
	
	private function getSeries($album_list) {
		$series = array();
		foreach ($album_list as $album)
			if (! isset($series[$album['series_id']]))
				$series[$album['series_id']] = $album['Series'];
		return $series;
	}
	
	function add() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'Rte';
		
		if (!empty($this->data)) {
			if ($this->Author->save($this->data)) {
				$this->smartFlash(__('The author has been added', true), 'info', array(
					'/authors/view/'.$this->Author->id => __('View author', true)
				), false);
			} else
				$this->generateLists();
		}
		
		// else just show the form
		$this->generateLists();
	}
	
	function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'Rte';
		
		if (empty($this->data)) {
			$this->Author->id = $id;
			$this->Author->recursive = -1;
			if (! $this->data = $this->Author->read())
				$this->errorNoSuchEntry(__('This author does not exist.', true));
			$this->generateLists();
		}
		
		else {
			if ($this->Author->save($this->data)) {
				$this->smartFlash(__('The author has been updated', true), 'info', array(
					'/authors/view/'.$this->Author->id => __('View author', true)
				));
			} else
				$this->generateLists();
		}
	}
	
	private function generateLists() {
		$this->set('imagesList', $this->Author->Image->find('list'));
	}
}

?>
