<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class CollectionsController extends AppController {
	public $scaffold;
	public $uses = array('Collection');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $components = array('PhpThumb');
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);
	
	public $paginate = array('limit' => 32, 'page' => 1,
		'fields' => array('Collection.id', 'Collection.name', 'Publisher.id', 'Publisher.name'),
		'order'=>array('Publisher.name' => 'ASC', 'Collection.name' => 'ASC'), 'recursive' => 1);
	
	public function index() {
		$this->helpers[] = 'MyPaginator';
		
		$this->paginate['limit'] = ($perColumn = Configure::read('Demyo.itemsPerIndexColumns')) * Configure::read('Demyo.columnsPerIndexPage');
		$this->set('perColumn', $perColumn);
		
		$this->Collection->unbindAll(array('belongsTo' => array('Publisher')), false);
		
		// Fix #57 - Collection index doesn't show Publishers with no Collection
		$this->Collection->belongsTo['Publisher']['type'] = 'RIGHT OUTER';
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Collection->findPageNumberStartsWith('Publisher.name', $this->params['named']['startsWith'], $this->paginate['limit']);
		}
		
		$collections =  $this->paginate('Collection');
		$this->set('collections', $collections);
	}

	public function view($id = null) {
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'Feed';
		$this->helpers[] = 'Rte';
		
		if ($id === null)
			$this->redirect('/collections');

		$this->Collection->id = $id;
		$this->Collection->contain(array('Image', 'Publisher.id', 'Publisher.name'));
		if (! $collection = $this->Collection->read())
			$this->errorNoSuchEntry(__('This collection does not exist.', true));
		if (! empty($collection['Image']['id']))
			$collection['Image']['thumb_url'] = $this->PhpThumb->getThumbnail($collection['Image']['url']);
		$collection['Collection']['album_count'] = $this->Collection->Album->getAlbumCount('Collection', $id);
		
		$this->set('collection', $collection);
	}
	
	public function add() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'Rte';
		
		if (!empty($this->data)) {
			if ($this->Collection->save($this->data)) {
				$this->smartFlash(__('The collection has been added', true), 'info', array(
					'/collections/view/'.$this->Collection->id => __('View collection', true)
				), false);
			} else
				$this->generateLists();
		}
		
		else {
			$this->generateLists();
			
			// Hack POST array to select default values
			if (isset($this->params['named']['toPublisher']))
				$this->data['Collection']['publisher_id'] = $this->params['named']['toPublisher'];
		}
	}
	
	public function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'Rte';
		
		if (empty($this->data)) {
			$this->Collection->id = $id;
			$this->Collection->recursive = -1;
			if (! $this->data = $this->Collection->read())
				$this->errorNoSuchEntry(__('This collection does not exist.', true));
			$this->generateLists();
		}
		
		else {
			if ($this->Collection->save($this->data)) {
				$this->smartFlash(__('The collection has been updated', true), 'info', array(
					'/collections/view/'.$this->data['Collection']['id'] => __('View collection', true)
				));
			} else
				$this->generateLists();
		}
	}
	
	private function generateLists() {
		$this->set('imagesList', $this->Collection->Image->find('list'));
		$this->set('publishersList', $this->Collection->Publisher->find('list'));
	}
	
	function fromPublisher($id = null) {
		$this->layout = 'ajax';
		
		if ($id != null)
			$this->set('collections', $this->Collection->find('list', array('conditions' => 'Collection.publisher_id='.$id)));
		else
			$this->set('collections', array());
	}
}

?>
