<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

App::Import('Model', 'AppModel');

/** Mockup model to have an easy access to the Database */
class InstallerDummyModel extends AppModel {
	public $useTable = false;
}

class InstallerComponent extends Object {
	public $components = array('DemyoConfiguration', 'Session');
	
	/* Install functions */
	
	public function shouldInstall() {
		return !file_exists(CONFIGS.'database.php');
	}
	
	public function createDatabaseConfig($host, $login, $password, $database, $prefix) {
		require (LIBS . 'model' . DS . 'datasources' . DS . DS . 'dbo_source' . '.php');
		require (LIBS . 'model' . DS . 'datasources' . DS . 'dbo' . DS . 'dbo_mysql' . '.php');
		
		$config = array(
			'driver' => 'mysql',
			'persistent' => true,
			'host' => $host,
			'login' => $login,
			'password' => $password,
			'database' => $database,
			'prefix' => $prefix,
			'encoding' => 'utf8',
		);
		
		@($dbo =& new DboMysql($config));
		if (! $dbo->isConnected())
			return __('Cannot connect to the database. Check your settings.', true);
		
		$content = var_export($config, true);
		$content = <<<EOS
<?php
class DATABASE_CONFIG {
var \$default = $content;
}
?>
EOS;
		if (! @file_put_contents(CONFIGS.'database.php', $content))
			return __('Failed to write database configuration. Check file permissions.', true);
		
		return true;
	}
	
	public function cleanDatabaseConfig() {
		return @unlink(CONFIGS.'database.php');
	}
	
	/** Create the tables.
	 * 
	 * @param $force
	 * @return 0 for success. 1 if tables already exist. 2 if creation failed.
	 */
	public function createTables($force = false) {
		$model =& new InstallerDummyModel();
		$baseDir = MODELS.'schema'.DS;
		$config = new DATABASE_CONFIG();
		
		$checkQuery = current($this->getQueries("${baseDir}check.sql", $config->default['prefix']));
		$check = @$model->query($checkQuery);
		
		if ($check)
			return 1;
		
		$queries = $this->getQueries("${baseDir}create-tables.sql", $config->default['prefix']);
		foreach ($queries as $query)
			if (@$model->query($query) === false)
				return 2;
		
		return 0;
	}
	
	/* Update functions */
	
	public function shouldUpdate() {
		$should = $this->Session->read('doUpdate') !==  'no'
			&& file_exists(CONFIGS.'demyo.php.old');
		$this->Session->delete('doUpdate'); // Only prevent update once
		return $should;
	}
	
	/** Updates the software to a new version.
	 * 
	 * @return 0 in case of success, 1 if merging configurations failed,
	 * 2 if update failed due to an SQL error. */
	public function update() {
		if (false === ($oldSchema = $this->mergeConfigs()))
			return 1;
		if (! $this->updateSchema($oldSchema))
			return 2;
		
		$this->Session->write('doUpdate', 'no');
		return 0;
	}
	
	/** Merges old and new configuration.
	 * 
	 * @return the schema version found in the old configuration, or false if
	 * merging failed.
	 */
	private function mergeConfigs() {
		$config = array();
		
		if (! file_exists(CONFIGS.'demyo.php.old'))
			return false;
		include(CONFIGS.'demyo.php.old');
		$oldConfig = $config;
		$oldSchema = (int) $config['Demyo.schemaVersion'];
		
		// Don't restore old version information
		unset($oldConfig['Demyo.versionAsString'],
			$oldConfig['Demyo.versionCodename'],
			$oldConfig['Demyo.versionAsFloat'],
			$oldConfig['Demyo.schemaVersion'],
			$oldConfig['Demyo.lastUpdateInfo']);
		
		$config = array();
		include(CONFIGS.'demyo.php');
		$newConfig = $config;
		
		$this->mergeConfigArrays($oldConfig, $newConfig);
		
		// Save to file
		$content = "<?php\n\n";
		foreach ($newConfig as $key => $value) {
			$content .= "\$config['$key'] = ".var_export($value, true).";\n";
		}
		$content .= "\n?>\n";
		if (file_put_contents(CONFIGS.'demyo.php', $content) === false)
			return false;
		if (! unlink(CONFIGS.'demyo.php.old'))
			return false;
		
		return $oldSchema;
	}
	
	private function mergeConfigArrays(&$old, &$new) {
		foreach ($new as $key => $value) {
			if (! isset($old[$key]))
				continue;
			
			if (is_array($value))
				$this->mergeConfigArrays($old[$key], $new[$key]);
			else
				$new[$key] = $old[$key];
		}
	}
	
	/** @return true in case of success, false if update failed due to an SQL error */
	private function updateSchema($from) {
		$to = Configure::read('Demyo.schemaVersion');
		$model =& new InstallerDummyModel();
		$baseDir = MODELS.'schema'.DS;
		$config = new DATABASE_CONFIG();
		
		for (; $from < $to; $from++) {
			$step = $from+1;
			$queries = $this->getQueries("${baseDir}${from}_to_${step}.sql", $config->default['prefix']);
			foreach ($queries as $query)
				if (@$model->query($query) === false)
					return false;
		}
		
		return true;
	}

	/** Creates a bunch of locale-specific image directories. */
	public function createImageDirectories() {
		$directories = Configure::read('Demyo-l10n.imageDirectories');
		foreach ($directories as $dir) {
			$dir = COLLECTION_IMAGES.$dir;
			if (! file_exists($dir)) {
				mkdir($dir);
				chmod($dir, 0777);
			}
		}
	}

	/* Update check functions */
	
	const UPDATE_CHECK_INTERVAL = 604800; // 7 days, in seconds
	const UPDATE_CHECK_URL = 'latest.json'; // Relative to website root
	
	public function shouldCheckForUpdate() {
		return time() - Configure::read('Demyo.lastUpdateCheck') > self::UPDATE_CHECK_INTERVAL;
	}
	
	/** Queries the Demyo website to check if updates are available.
	 * 
	 * @return FALSE on failure of if the software is up-to-date. If an update
	 * is available, an associative array is returned (keys: versionAsFloat,
	 * versionAsString, versionCodename, releasePage)
	 */
	public function getUpdateInformation() {
		$this->markUpdateCheck(); // Do it before any error checking so that we
		// don't slow everything until we finally have some valid information
		
		if (($content = file_get_contents(Configure::read('Demyo.websiteRoot').self::UPDATE_CHECK_URL)) === false)
			return false;
		
		$json = json_decode($content, true);
		if ($json === null || $json === false)
			return false;
		
		$updateInfo = $json['default'];
		$lang = Configure::read('Demyo-l10n.websiteLang');
		if (isset($json[$lang]))
			$updateInfo = array_merge($json['default'], $json[$lang]);
		
		// Check if we need to update
		if ($updateInfo['versionAsFloat'] > Configure::read('Demyo.versionAsFloat')) {
			// Save update info
			Configure::write('Demyo.lastUpdateInfo', $updateInfo);
			$this->DemyoConfiguration->saveToFile();
			return $updateInfo;
		}
		return false;
	}
	
	/** Saves the current time as the last update check time. */
	public function markUpdateCheck() {
		Configure::write('Demyo.lastUpdateCheck', time());
		$this->DemyoConfiguration->saveToFile();
	}
	
	/* Utility functions */
	
	private function getQueries($filename, $prefix) {
		$queries = file_get_contents($filename);
		$queries = str_replace('__PREFIX__', $prefix, $queries);
		$queries = explode(";\n", $queries);
		
		$queries_real = array();
		foreach ($queries as $query) {
			$query = trim($query);
			if (! empty($query))
				$queries_real[] = $query;
		}
		
		return $queries_real;
	}
}
?>