<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class SpreadsheetImportExportComponent extends Object {
	private $warnings;
	private $controller;
	
	/***************************** CSV Import ********************************/
	
	/** Imports data from a CSV file into the database.
	 * Expected format :
	 * series;cycle;number;title;artist;writer;colorist;publisher;collection;first_edition;this_edition;purchase_date;isbn;binding;pages;summary;comment;price
	 * @param ProgressReporter $reporter
	 */
	// TODO: we can probably cache some results
	public function importFromCSV(&$controller, $filename, $reporter) {
		$csv = fopen($filename, 'r');
		$total = 0;
		
		$this->warnings = array();
		$this->controller =& $controller;
		
		// Delete all
		$controller->Album->deleteAll('1 = 1', false);
		$controller->AlbumsBorrower->deleteAll('1 = 1', false);
		$controller->Author->deleteAll('1 = 1', false);
		$controller->Binding->deleteAll('1 = 1', false);
		$controller->Borrower->deleteAll('1 = 1', false);
		$controller->Collection->deleteAll('1 = 1', false);
		$controller->Derivative->deleteAll('1 = 1', false);
		$controller->DerivativeType->deleteAll('1 = 1', false);
		$controller->Image->deleteAll('1 = 1', false);
		$controller->Publisher->deleteAll('1 = 1', false);
		$controller->Series->deleteAll('1 = 1', false);
		$controller->Source->deleteAll('1 = 1', false);
		$controller->Tag->deleteAll('1 = 1', false);

		// We'll build this step by step
		$adata = null;

		$curDate = date("Y-m-d");

		while(($line = fgetcsv($csv, 0, ';', '"')) !== false) {
			$total++;
			unset($adata);
			// Load fields
			list($series, $cycle, $number, $title, $artist, $writer,
				$colorist, $publisher, $collection, $first_edition,
				$this_edition, $purchase_date, $isbn, $binding, $pages,
				$summary, $comment, $price) = $line;

			// Fix some of them (not really a good idea to always do this.
			// This should be disabled unless explicitely set.
			/*$series = ucwords(strtolower($series));
			$title = ucwords(strtolower($title));
			$artist = ucwords(strtolower($artist));
			$writer = ucwords(strtolower($writer));
			$colorist = ucwords(strtolower($colorist));
			$publisher = ucwords(strtolower($publisher));
			$collection = ucwords(strtolower($collection));*/

			// Set trivial information
			foreach (array('cycle', 'title', 'number', 'isbn', 'summary', 'comment') as $trivial)
				if (strlen(${$trivial}) > 0)
					$adata['Album'][$trivial] = ${$trivial};
			if (strlen($price)>0 && $price > 0) {
				$adata['AlbumPrice'][0]['price'] = $price;
				$adata['AlbumPrice'][0]['date'] = $curDate;
			}
			if (strlen($pages)>0 && $pages > 0)
				$adata['Album']['pages'] = $pages;

			// Check names
			list($artist_name, $artist_fname) = $this->parseName($artist);
			list($writer_name, $writer_fname) = $this->parseName($writer);
			list($colorist_name, $colorist_fname) = $this->parseName($colorist);
			
			// Check dates
			if (strlen($first_edition = $this->fixDate($first_edition))>0)
				$adata['Album']['first_edition'] = $first_edition;
			if (strlen($this_edition = $this->fixDate($this_edition))>0)
				$adata['Album']['this_edition'] = $this_edition;
			if (strlen($purchase_date = $this->fixDate($purchase_date))>0)
				$adata['Album']['purchase_date'] = $purchase_date;

			// Search for items or insert them {{{
			// Search for publisher, or insert it
			if (strlen($publisher) > 0) {
				$publisher_id = $this->getIdOrInsert('Publisher',
					array('name'=>$publisher));
				$adata['Album']['publisher_id'] = $publisher_id;
			} else {
				$this->warning($total, $line, "Empty publisher");
				continue;
			}

			// Search for collection, or insert it
			if (strlen($collection) > 0) {
				$collection_id = $this->getIdOrInsert('Collection',
					array('name'=>$collection, 'publisher_id'=>$publisher_id));
				$adata['Album']['collection_id'] = $collection_id;
			} // Drop empty
			
			// Search for binding, or insert it
			if (strlen($binding) > 0) {
				$binding_id = $this->getIdOrInsert('Binding',
					array('name'=>$binding));
				$adata['Album']['binding_id'] = $binding_id;
			} // Drop empty

			// Search for series, or insert it
			if (strlen($series) > 0) {
				$series_id = $this->getIdOrInsert('Series',
					array('name'=>$series));
				$adata['Album']['series_id'] = $series_id;
			} else {
				$this->warning($total, $line, "Empty series");
				continue;
			}
			
			// Search for artist, or insert it
			if ($artist_name != '') {
					$artist_id = $this->getIdOrInsert('Artist',
						array('name'=>$artist_name, 'fname'=>$artist_fname));
				$adata['Artist']['Artist'] = array($artist_id);
			} // Drop empty
			
			// Search for writer, or insert it
			if ($writer_name != '') {
					$writer_id = $this->getIdOrInsert('Writer',
						array('name'=>$writer_name, 'fname'=>$writer_fname));
				$adata['Writer']['Writer'] = array($writer_id);
			} // Drop empty
			
			// Search for colorist, or insert it
			if ($colorist_name != '') {
					$colorist_id = $this->getIdOrInsert('Colorist',
						array('name'=>$colorist_name, 'fname'=>$colorist_fname));
				$adata['Colorist']['Colorist'] = array($colorist_id);
			} // Drop empty
			// }}}

			$controller->Album->create();
			if ($controller->Album->save($adata) === false)
				$this->warning($total, $line, 'Failed insertion for unknown reason');
			
			if (!($total%50))
				$reporter->setCurrent($total);
		}
		fclose($csv);
		$reporter->setCurrent($total);

		return $this->warnings;
	}
	
	/************************ Utilities for import ***************************/
	
	private function parseName($name) {
		$name = trim($name);
		
		$words = explode(' ',$name); // Guaranteed to return non-false
		if (sizeof($words) == 1) {
			$lastName = $words[0];
			$firstName = '';
		} else if (sizeof($words) == 2) {
			$lastName = $words[0];
			$firstName = $words[1];
		} else {
			$lastName = $name;
			$firstName = '';
		}
		
		return array($lastName, $firstName);
	}

	/** Returns a fixed date, or an empty string if an error occurred
	 * Expects YYYY-MM-DD, YYYY-MM or YYYY
	 */
	private function fixDate($date) {
		if (preg_match('/^\\d{4}-\\d{2}-\\d{2}$/', $date))
			return $date;
		else if (preg_match('/^\\d{4}-\\d{2}$/', $date))
			return "$date-01";
		else if (preg_match('/^\\d{4}$/', $date))
			return "$date-01-01";
		else if (preg_match('+^(\\d{2})/(\\d{2})/(\\d{4})$+', $date, $matches))
			return "{$matches[3]}-{$matches[2]}-{$matches[1]}";
		else
			return '';
	}

	private function getIdOrInsert($modelName, $fields) {
		$this->controller->$modelName->recursive = -1;
		$entry = $this->controller->$modelName->find('first', array('conditions' => $fields, 'fields' => array('id')));
		if ($entry === false) { // Insert
			foreach($fields as $key=>$value)
				$data[$modelName][$key] = $value;
			$this->controller->$modelName->create();
			$this->controller->$modelName->save($data, false);
			return $this->controller->$modelName->getLastInsertID();
		} else
			return $entry[$modelName]['id'];
	}

	private function warning($number, $line, $message, $echo = false) {
		$line = implode(';',$line);
		$str = "Warning: line $number dropped ~ $message (line was $line)";
		if ($echo)
			debug($str);
		$this->warnings[] = $str;
	}
	
	/***************************** CSV Export ********************************/
	
	public function exportToCSV($controller, $options) {
		$albums = $this->getExportDataSet($controller, $options);
		
		$output = '';
		foreach ($albums as $album) {
			$this->fixAlbumFields($album);
			
			$output .= $this->getCSVLine(array($album['Series']['name'],
				$album['Album']['cycle'],$album['Album']['number'],$album['Album']['title'],
				implode(', ', Set::extract('/Writer/meta_name', $album)),
				implode(', ', Set::extract('/Artist/meta_name', $album)),
				implode(', ', Set::extract('/Colorist/meta_name', $album)),
				$album['Publisher']['name'],$album['Collection']['name'],
				$album['Album']['first_edition'],$album['Album']['this_edition'],
				$album['Album']['purchase_date'],$album['Album']['isbn'],
				$album['Binding']['name'],$album['Album']['pages'],
				$album['Album']['summary'],$album['Album']['comment'],
				$album['Album']['purchase_price']
			));
		}
		
		return array('demyo.csv', $output);
	}
	
	private function getCSVLine(array $fields, $delimiter = ';', $enclosure = '"') {
		foreach ($fields as &$field)
			$field = $enclosure . str_replace($enclosure, $enclosure . $enclosure, $field) . $enclosure;
		
		return implode($delimiter, $fields)."\n";
	}
	
	/***************************** XLS Export ********************************/
	
	public function exportToXLS($controller, $options) {
		return $this->exportToExcel($controller, $options, 'Excel5', 'demyo.xls');
	}

	public function exportToXLSX($controller, $options) {
		return $this->exportToExcel($controller, $options, 'Excel2007', 'demyo.xlsx');
	}

	private function exportToExcel($controller, $options, $version, $filename) {
		set_include_path(get_include_path() . PATH_SEPARATOR . APP . 'vendors' . DS . 'PHPExcel');
		App::import('Vendor', 'PHPExcel', array('file' => 'PHPExcel/PHPExcel.php'));
		App::import('Vendor', 'PHPExcel_IOFactory', array('file' => 'PHPExcel/PHPExcel/IOFactory.php'));
		
		$phpExcel = new PHPExcel();
		$phpExcel->getProperties()->setCreator("Demyo");
		$phpExcel->getProperties()->setLastModifiedBy("Demyo");
		$phpExcel->getProperties()->setTitle(__("List of albums", true));
		$phpExcel->getProperties()->setSubject(__("List of albums", true));
		$phpExcel->getProperties()->setDescription(__("List of albums generated by Demyo.", true));
		
		$phpExcel->setActiveSheetIndex(0);
		$phpExcelSheet =& $phpExcel->getActiveSheet();
		$phpExcelSheet->setTitle(__('Albums', true));

		// Column width. Disabled: doesn't work on WinMo
		//for ($a = 'A'; $a < 'R'; $a++)
		//	$phpExcelSheet->getColumnDimension($a)->setAutoSize(true);
		
		// Header
		$this->addSheetLine($phpExcelSheet, array(__('Series', true),
				__('Cycle', true),__('Number', true),__('Title', true),
				__('Writers', true),
				__('Artists', true),
				__('Colorists', true),
				__('Publisher', true),__('Collection', true),
				__('First edition', true),__('Edition', true),
				__('Purchase date', true),__('ISBN', true),
				__('Binding', true),__('Pages', true),
				__('Summary', true),__('Comment', true)
			), 1, true);

		// Content
		$albums = $this->getExportDataSet($controller, $options);
		$line = 2;
		foreach ($albums as $album) {
			$this->fixAlbumFields($album);
			
			$this->addSheetLine($phpExcelSheet, array($album['Series']['name'],
				$album['Album']['cycle'],$album['Album']['number'].$album['Album']['number_suffix'],$album['Album']['title'],
				implode(', ', Set::extract('/Writer/meta_name', $album)),
				implode(', ', Set::extract('/Artist/meta_name', $album)),
				implode(', ', Set::extract('/Colorist/meta_name', $album)),
				$album['Publisher']['name'],$album['Collection']['name'],
				$album['Album']['first_edition'],$album['Album']['this_edition'],
				$album['Album']['purchase_date'],$album['Album']['isbn'],
				$album['Binding']['name'],$album['Album']['pages'],
				$album['Album']['summary'],$album['Album']['comment']
			), $line++);
		}

		// Work around http://bugs.php.net/bug.php?id=42473
		$tempFile = tempnam(TMP, $filename);
		$phpExcelWriter = PHPExcel_IOFactory::createWriter($phpExcel, $version);
		$phpExcelWriter->save($tempFile);
		$output = file_get_contents($tempFile);
		unlink($tempFile);
		return array($filename, $output);
	}
	
	private function addSheetLine(&$phpExcelSheet, array $fields, $line, $isHeader = false) {
		$column = 'A';
		foreach ($fields as &$field) {
			if ($isHeader) {
				$phpExcelSheet->getStyle($column.$line)->getFont()->setBold(true);
				$phpExcelSheet->getStyle($column.$line)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$phpExcelSheet->getStyle($column.$line)->getBorders()->getBottom()->setBorderStyle(PHPExcel_Style_Border::BORDER_THIN);
			}
			$phpExcelSheet->getStyle($column.$line)->getFont()->getColor()->setARGB('FF000000');
			$phpExcelSheet->setCellValueExplicit($column.$line, $field);
			$column++;
		}
	}
	
	/************************ Utilities for export ***************************/
	
	private function getExportDataSet($controller, $options) {
		$conditions = array();
		if (! empty($options['wishlist']))
			$conditions = array('Album.wishlist' => true);
		
		$controller->Album->contain(array(
			'Series.name',
			'Writer.name', 'Writer.fname', 'Writer.nickname',
			'Artist.name', 'Artist.fname', 'Artist.nickname',
			'Colorist.name', 'Colorist.fname', 'Colorist.nickname',
			'Publisher.name',
			'Collection.name',
			'Binding.name'
		));
		
		return $controller->Album->find('all', array('conditions' => $conditions,
			'order' => array('Series.name' => 'ASC', 'Album.cycle' => 'ASC',
				'Album.number' => 'ASC', 'Album.number_suffix' => 'ASC')));
	}
	
	private function fixAlbumFields(&$album) {
		foreach (array('first_edition', 'this_edition', 'purchase_date') as $dateField)
			if ($album['Album'][$dateField] === '0000-00-00')
				$album['Album'][$dateField] = '';
		foreach (array('pages', 'purchase_price') as $numberField)
			if ($album['Album'][$numberField] == 0)
				$album['Album'][$numberField] = '';
	}
}
?>
