<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class ConfiguratorController extends AppController {
	public $name = 'Configurator';
	
	public $uses = array();
	public $components = array('DemyoConfiguration', 'ThemeManager');
	public $helpers = array('Form', 'jQuery');
	protected $smartFlashActions = array();
	
	public function edit() {
		if (empty($this->data)) {
			$locales = array();
			$localesDir = new DirectoryIterator(APP.'locale'.DS);
			
			// Build an array of available locales
			// Get locale names
			App::import('core', 'L10n');
			$L10n = new L10n(); 
			$reversedCatalog = array();
			foreach ($L10n->catalog() as $lang) {
				$reversedCatalog[$lang['locale']] = $lang['language'];
			}
			
			// Get actually available locales
			foreach ($localesDir as $entry)
				if ($entry->isDir()) {
					$name = $entry->getFileName();
					if ($name{0} !== '.') // Skip ., .. and all hidden directories
						$locales[$name] = $reversedCatalog[$name];
				}
			
			$this->set('locales', $locales);
			$this->set('availableThemes', $this->ThemeManager->getAvailableThemes());
			$this->set('chartMethods', array('Flash' => 'Flash', 'Google' => 'Google'));
			$this->set('config', Configure::read('Demyo'));
		} else {
			// Fetch current config
			$currentConfig = Configure::read('Demyo');
			// If theme changed, clear the cache
			if ($currentConfig['theme'] != $this->data['theme']) {
				$this->ThemeManager->clearCSSCache();
				// Also, load the new theme already. Else, we'll regenerate
				// a compressed CSS for the old theme
				$this->theme = $this->data['theme'];
			}
			// Load config from POST data
			$this->loadFromData($currentConfig, $this->data);
			// Save
			$this->DemyoConfiguration->saveToFile($currentConfig);
			
			return $this->flash(__('Configuration saved', true), '/configurator/edit');
		}
	}
	
	public function save($key, $value) {
		Configure::write("Demyo.$key", $value);
		$this->DemyoConfiguration->saveToFile();
		return $this->flash(__('Configuration saved', true), '/');
	}
	
	private function loadFromData(&$current, &$data) {
		foreach ($current as $key => &$subCurrent) {
			if (is_array($subCurrent))
				$this->loadFromData($subCurrent, $data[$key]);
			else if (is_bool($current[$key]))
				$subCurrent = isset($data[$key]) && ($data[$key] === true || $data[$key] === 'on');
			else if (is_int($current[$key]))
				$subCurrent = isset($data[$key])?(int) $data[$key]:$subCurrent;
			else
				$subCurrent = isset($data[$key])?$data[$key]:$subCurrent;
		}
	}
}

?>