<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class DerivativesController extends AppController {
	public $scaffold;
	public $uses = array('Derivative');
	public $components = array('PhpThumb', 'RequestHandler');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $paginate = array('limit' => 45,
		'page' => 1,
		'order'=>array('Series.name' => 'ASC', 'Album.cycle' => 'ASC', 'Album.number' => 'ASC', 'Album.number_suffix' => 'ASC', 'Source.name' => 'ASC'),
		'fields' => array(
			'Derivative.id', 'Derivative.album_id', 'Derivative.colors', 'Derivative.source_id', 'Derivative.number', 'Derivative.total',
			'Series.id', 'Series.name',
			'Album.title', 'Album.first_edition',
			'DerivativeType.*',
			'Source.id', 'Source.name',
			'Artist.id', 'Artist.name', 'Artist.fname', 'Artist.nickname'
		),
		'recursive' => 1);
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'labels' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);
		
	
	public function index() {
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'MyPaginator';
		$this->helpers[] = 'Text';
		
		$this->paginate['limit'] = Configure::read('Demyo.Gallery_thumbsPerPage');
		
		$isAjax = isset($this->params['named']['ajax']); // Bug #193
		$this->set('isAjax', $isAjax); 
		if ($isAjax) {
			$this->layout = 'ajax';
		}
		
		$filters = $this->getFilters(array('fromSeries', 'fromAlbum', 'fromArtist', 'fromType', 'fromSource'));
		extract($filters);
		
		$scope = array();
		if ($fromSeries !== null) {
			array_push($scope, array('Series.id' => $fromSeries));
			$this->set('title_for_layout', __s('Viewing derivatives from %s', $this->Derivative->Series->getName($fromSeries)));
		}
		if ($fromAlbum !== null) {
			array_push($scope, array('Album.id' => $fromAlbum));
			$this->set('title_for_layout', __s('Viewing derivatives from %s', $this->Derivative->Album->getName($fromAlbum)));
		}
		if ($fromArtist !== null) {
			$this->loadModel('Author'); // Load it, we only have Derivative->Artist, and it's buggy
			array_push($scope, array('Artist.id' => $fromArtist));
			$this->set('title_for_layout', __s('Viewing derivatives from %s', $this->Author->getName($fromArtist)));
		}
		if ($fromType !== null) {
			array_push($scope, array('DerivativeType.id' => $fromType));
			$this->set('title_for_layout', __s('Viewing derivatives of type \'%s\'', $this->Derivative->DerivativeType->getName($fromType)));
		}
		if ($fromSource !== null) {
			array_push($scope, array('Source.id' => $fromSource));
			$this->set('title_for_layout', __s('Viewing derivatives from \'%s\'', $this->Derivative->Source->getName($fromSource)));
		}

		if (empty($scope) || $isAjax)
			$this->set('title_for_layout', __('Derivatives', true));
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Derivative->findPageNumberStartsWith('Series.name', $this->params['named']['startsWith'], $this->paginate['limit'], $scope);
		}
		
		$derivs =  $this->paginate('Derivative', $scope);
		foreach ($derivs as &$deriv)
			foreach ($deriv['Image'] as &$image)
				$image['thumb_url'] = $this->PhpThumb->getThumbnail($image['url']);
		
		$this->params['pass'] = array_filter($filters);
		$this->set('derivatives', $derivs);
	}
	
	function view($id = null) {
		$this->helpers[] = 'Chart';
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'Rte';
		
		if ($id === null)
			$this->redirect('/derivatives');

		$this->Derivative->id = $id;
		$this->Derivative->contain(array(
			'Series.id', 'Series.name',
			'Album.id', 'Album.title',
			'DerivativeType', 'Image',
			'Artist.id', 'Artist.name', 'Artist.fname', 'Artist.nickname',
			'Source.id', 'Source.name',
			'DerivativePrice'
		));
		if (! $derivative = $this->Derivative->read())
			$this->errorNoSuchEntry(__('This derivative does not exist.', true));
		
		foreach($derivative['Image'] as &$image)
				$image['thumb_url'] = $this->PhpThumb->getThumbnail($image['url']);
		
		$this->set('derivative', $derivative);
	}
	
	function add() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'Rte';
		
		if (!empty($this->data)) {
			$this->saveDerivative(array(
				'message' => __('The derivative has been added', true),
				'gotos' => array(
					'/derivatives/view/$_MODEL_ID_$' => __('View derivative', true))
				), false);
		}
		
		else {
			$this->generateLists();
			
			// Hack post array to auto-select some fields
			if (isset($this->params['named']['toAlbum'])) {
				$album = $this->params['named']['toAlbum'];
				$this->Derivative->Album->contain('Artist');
				$data = $this->Derivative->Album->find('first', array('conditions' => array('Album.id' => $album)));
				$this->data['Derivative']['series_id'] = $data['Album']['series_id'];
				$this->data['Derivative']['album_id'] = $album;
				$this->data['Derivative']['purchase_date'] = ($data['Album']['purchase_date'] === '0000-00-00')?'':$data['Album']['purchase_date'];
				$this->data['Derivative']['artist_id'] = empty($data['Artist'])?0:$data['Artist'][0]['id'];
			}
			else if (isset($this->params['named']['toSeries']))
				$this->data['Derivative']['series_id'] = $this->params['named']['toSeries'];
		}
	}
	
	function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'Rte';
		
		if (empty($this->data)) {
			$this->Derivative->id = $id;
			// Reminder: some fields are needed for the page title
			$this->Derivative->contain('DerivativeType.name', 'Series.name', 'Source.name', 'Image.id', 'DerivativePrice');
			if (! $this->data = $this->Derivative->read())
				$this->errorNoSuchEntry(__('This derivative does not exist.', true));
			
			// Fix date, see #34
			if (isset($this->data['Derivative']['purchase_date']) && $this->data['Derivative']['purchase_date'] === '0000-00-00')
				$this->data['Derivative']['purchase_date'] = '';
				
			$this->generateLists($this->data['Derivative']['series_id']);
		}
		
		else {
			$this->saveDerivative(array(
				'message' => __('The derivative has been updated', true),
				'gotos' => array(
					'/derivatives/view/'.$this->data['Derivative']['id'] => __('View derivative', true))
				));
		}
	}
	
	private function saveDerivative($flashOptions, $redirectToPrevious = true) {
		// Set source
		$this->data['Derivative']['source_id'] = $this->Derivative->Source->getOrAdd($this->data['Source']['name']);
		unset($this->data['Source']);
		
		// Don't trust given series_id; fetch it from the album
		if (! empty($this->data['Derivative']['album_id'])) 
			$this->data['Derivative']['series_id'] = current(current(
				$this->Derivative->Album->find('first', array('fields' => 'series_id',
					'conditions' => array('id' => $this->data['Derivative']['album_id']),
					'recursive' => -1))));
		
		// Check prices
		if (! empty($this->data['DerivativePrice']))
			foreach($this->data['DerivativePrice'] as $key => $value) {
				if (empty($value['date']) || empty($value['price']))
					unset($this->data['DerivativePrice'][$key]);
			}
		
		$this->Derivative->set($this->data);
		if ($this->Derivative->validates() === true) {
			// See note about this in AlbumsController
			if (! empty($this->data['Derivative']['id'])) // Obviously, this step is useless for new derivatives
				$this->Derivative->DerivativePrice->deleteForDerivative($this->data['Derivative']['id']);
			
			if ($this->Derivative->saveAll($this->data, array('validate' => false))) {
				extract($flashOptions);
				$this->smartFlash($message, 'info', $gotos, $redirectToPrevious);
			} else
				$this->generateLists(); 
		} else
			$this->generateLists();
	}
	
	private function generateLists($seriesID = -1) {
		$seriesList = $this->Derivative->Series->find('list');
		
		if ($seriesID === -1)
			$seriesID = current(array_keys($seriesList));
		
		$this->set('seriesList', &$seriesList);
		$this->loadModel('Author'); // Load it, we only have Derivative->Artist, and it's buggy
		$this->set('authorsList', $this->Author->find('list'));
		$this->set('typesList', $this->Derivative->DerivativeType->find('list'));
		$this->set('imagesList', $this->Derivative->Image->find('list'));
		$this->set('albumsList', $this->Derivative->Album->find('list', array('conditions' => 'Album.series_id='.$seriesID)));
	}
	
	public function labels() {
		$this->helpers[] = 'AlbumTools';
		
		$this->layout = 'printable';
		
		$this->Derivative->unbindModel(array('hasAndBelongsToMany' => array('Image')));
		$this->set('derivatives', $this->Derivative->find('all', array('fields' => array(
				'Series.name',
				'Album.cycle', 'Album.number', 'Album.number_suffix', 'Album.title', 'Album.first_edition',
				'Artist.name', 'Artist.fname', 'Artist.nickname',
				'Source.name',
				'DerivativeType.name',
				'Derivative.number', 'Derivative.total', 'Derivative.authors_copy', 'Derivative.restricted_sale'),
			 'recursive' => 1, 'order' => $this->paginate['order'])));
	}
}

?>
