<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class ImagesController extends AppController {
	public $scaffold;
	public $uses = array('Image');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $components = array('File', 'PhpThumb');
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);
	
	public function index() {
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'MyPaginator';
		
		$images = array();
		
		$this->set('doPaginate', false);

		// Note: filters are not paginated here
		if (isset($this->params['named']['fromSeries'])) {
			$this->loadModel('Series');
			$images = $this->Series->getImages($this->params['named']['fromSeries']);
		} else if (isset($this->params['named']['fromAlbum']))
			$images = $this->Image->Album->getImages($this->params['named']['fromAlbum']);
		else {
			$limit = Configure::read('Demyo.Gallery_thumbsPerPage');
			if (isset($this->params['named']['startsWith']))
				$page = $this->Image->findPageNumberStartsWith('description', $this->params['named']['startsWith'], $limit);
			else
				$page = isset($this->params['named']['page'])?$this->params['named']['page']:1;
			
			$images = Set::extract($this->Image->find('all',
				array('limit' => $limit, 'page' => $page, 'recursive' => -1)),
				'/Image/.');
			
			// Prepare paginator
			$this->prepareManualPagination('Image', $page, $limit, $this->Image->find('count'));
		}

		foreach($images as &$image) {
			$image['thumb_url'] = $this->PhpThumb->getThumbnail($image['url']);
		}
		
		$this->set('images', $images);
	}
	
	public function view($id = null) {
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'List';
		
		if ($id === null)
			$this->redirect('/images/');
		
		$this->Image->id = $id;
		$this->Image->recursive = 2;
		$this->Image->contain(
			array(
			'Album.id', 'Album.title',
			'Derivative.id', 'Derivative.Source',
				'Derivative.DerivativeType.name',
				'Derivative.Series.name'));
		if (! $image = $this->Image->read())
			$this->errorNoSuchEntry(__('This image does not exist.', true));
		$image['Image']['thumb_url'] = $this->PhpThumb->getThumbnail($image['Image']['url']);
		$this->set('image', $image['Image']);
		$this->set('albums', $image['Album']);
		$this->set('derivatives', $image['Derivative']);
	}
	
	public function add() {
		$source = empty($this->params['named']['from'])?'disk':$this->params['named']['from'];
		$this->action = $action = 'addFrom'.ucfirst($source);
		return $this->$action();
	}
	
	public function addMultiple() {
		$source = empty($this->params['named']['from'])?'disk':$this->params['named']['from'];
		$this->action = $action = 'addMultipleFrom'.ucfirst($source);
		return $this->$action();
	}

	public function addFromDisk() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		
		$this->set('title_for_layout', __('Add an image from disk', true));
		
		if (!empty($this->data['Image']['file'])) {
			$destFileName = str_replace(' ', '_', $this->data['Image']['file']['name']);

			if (!empty($this->data['Image']['folder'])) {
				$destFolder = $this->data['Image']['folder'].DS;
			}

			$destPath = COLLECTION_IMAGES.$destFolder.$destFileName;
			if (file_exists($destPath)) {
				$this->Image->validationErrors['folder'] = __('A file with the same name already exists in that folder', true);
				return;
			}
			rename($this->data['Image']['file']['tmp_name'], $destPath);
			chmod($destPath, 0666);

			$this->data['Image']['is_local'] = true;
			$this->data['Image']['url'] = $destFolder.$destFileName;
			unset($this->data['Image']['folder']);
			unset($this->data['Image']['file']);

			if ($this->Image->save($this->data)) {
				$this->smartFlash(__('The image has been added', true), 'info', array(
					'/images/view/'.$this->Image->id => __('View image', true)
				), false);
			}
		}
	}

	public function addFromDirectory() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		
		if (!empty($this->data)) {
			$this->data['Image']['is_local'] = (preg_match('@^(http|https|ftp)://.*@', $this->data['Image']['url']) === 0);
			
			if ($this->Image->save($this->data)) {
				$this->smartFlash(__('The image has been added', true), 'info', array(
					'/images/view/'.$this->Image->id => __('View image', true)
				), false);
			}
		}
	}
	
	public function addMultipleFromDirectory() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'HelpBrowser';

		$this->set('title_for_layout', __('Add multiple images', true));
		
		$step = empty($this->params['named']['step'])?null:$this->params['named']['step'];
		if (empty($this->data)) {
			$knownUrls = Set::extract($this->Image->find('all', array('fields' => array('is_local', 'url'), 'recursive' => -1)), '/Image/url');
			$localPaths = array();
			
			// List the images with paths relative to the img/ dir (not what is in the database, but fixed by the model)
			$stripLength = strlen(COLLECTION_IMAGES);
			foreach ($this->File->getDirectoryIterator(COLLECTION_IMAGES, true) as $file) {
				if (!$file->isDir() && $this->File->fileMatches($file, '/\.(gif|png|bmp|jpg|jpeg)$/i')) {
					$localPaths[] = COLLECTION_IMAGES_DIRNAME.DS.substr($file->getPathname(), $stripLength);
				}
			}
			
			$missing = array_diff($localPaths, $knownUrls);

			// Fix the paths to not include the COLLECTION_IMAGES_DIRNAME
			$stripLength = strlen(COLLECTION_IMAGES_DIRNAME.DS);
			foreach ($missing as &$path) {
				$path = substr($path, $stripLength);
			}
			natcasesort($missing);

			$this->set('missing', $missing);
			$this->action = 'add_multiple_from_directory_select'; // Override view name
		}
		
		else if ($step == 'descriptions') {
			$images = array();
			foreach($this->data['Image'] as $image => $nothing) {
				$images[] = array('url' => $image, 'description' => $this->imagePathToName($image));
			}
			$this->set('images', $images);
			$this->data['Image'] = $images;
			$this->action = 'add_multiple_from_directory_descriptions'; // Override view name
		}
		
		else if ($step == 'save') {
			$this->Image->saveAll($this->data['Image']);
			$this->flash(__('The images have been added', true), '/images/');
		}
	}
	
	public function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'jQuery';
		
		if (empty($this->data)) {
			$this->Image->id = $id;
			$this->Image->recursive = -1;
			if (! $this->data = $this->Image->read())
				$this->errorNoSuchEntry(__('This image does not exist.', true));
			$this->data['Image']['url'] = str_replace(COLLECTION_IMAGES_DIRNAME.DS, '', $this->data['Image']['url']);
		}
		
		else {
			$this->data['Image']['is_local'] = (preg_match('@^(http|https|ftp)://.*@', $this->data['Image']['url']) === 0);
			
			if ($this->Image->save($this->data)) {
				$this->smartFlash(__('The image has been updated', true), 'info', array(
					'/images/view/'.$this->data['Image']['id'] => __('View image', true)
				));
			}
		}
	}
	
	public function random($number = 1) {
		$this->layout = 'raw';
		$this->helpers[] = 'Colorbox';
		
		$images = Set::extract('/Image/.', $this->Image->getRandomImages($number));
		foreach ($images as &$image)
			$image['thumb_url'] = $this->PhpThumb->getThumbnail($image['url']);
		$this->set('images', $images);
	}
	
	private function imagePathToName($path) {
		return str_replace('_', ' ', pathinfo($path, PATHINFO_FILENAME));
	}
	
	// Required route:
	// Router::connect('/images/getThumbnail/:thumbnail', array('controller' => 'images', 'action' => 'getThumbnail'), array('thumbnail' => '.*'));
	public function getThumbnail() {
		$this->redirect(
			$this->PhpThumb->getThumbnailFromURL($this->params['thumbnail']));
	}
	
	public function cleanThumbnails() {
		$this->PhpThumb->clean();
		if (isset($_SERVER['HTTP_REFERER']))
			$url = $_SERVER['HTTP_REFERER'];
		else
			$url = Router::url('/', true);
		$this->flash(__('Thumbnails have been cleaned, they will be regenerated automatically', true), $url);
	}
}

?>
