<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class InstallerController extends AppController {
	public $uses = array();
	public $components = array('Installer');

	public function install() {
		if (! $this->Installer->shouldInstall())
			return $this->flash(__('Demyo looks already installed', true), '/');
		
		$message = true;
		if (! empty($this->data)) {
			$message = $this->Installer->createDatabaseConfig($this->data['host'], $this->data['user'], $this->data['password'], $this->data['database'], $this->data['prefix']);
		}
		
		if (empty($this->data) || $message !== true) {
			$this->set('message', $message);
			$connectionData = array_merge(array('host' => 'localhost', 'user' => 'demyo', 'password' => 'password', 'database' => 'demyo', 'prefix' => 'demyo_'), (array)$this->data);
			$this->set('host', $connectionData['host']);
			$this->set('user', $connectionData['user']);
			$this->set('password', $connectionData['password']);
			$this->set('database', $connectionData['database']);
			$this->set('prefix', $connectionData['prefix']);
		}
		else {
			// Fake an update check: we don't want to nag the user directly if he's
			// just installed the software.
			$this->Installer->markUpdateCheck();

			$this->Installer->createImageDirectories();

			return $this->redirect('/installer/create'); // Redirect
		}
	}
	
	public function create() {
		if ($this->Installer->shouldInstall())
			return $this->redirect('/installer/install');
		
		$ret = $this->Installer->createTables();
		
		switch ($ret) {
			case 0:
				return $this->flash(__('Demyo was successfully installed', true), '/');
			case 1:
				$this->set('error', __('The tables are already created. You should uninstall the old ones first.', true));
				$this->Installer->cleanDatabaseConfig();
				return;
			case 2:
				$this->set('error', __('Failed to create tables. Check permissions for the database user.', true));
				$this->Installer->cleanDatabaseConfig();
				return;
		}
	}
	
	public function update() {
		if (! $this->Installer->shouldUpdate())
			return $this->flash(__('No update to do', true), '/');
		
		$ret = $this->Installer->update();
		
		switch ($ret) {
			case 0:
				return $this->flash(__('Update successful', true), '/');
			case 1:
				$this->set('error', __('Failed to restore old configuration. If you followed the exact update procedure, please file a bug report.', true));
				return;
			case 2:
				$this->set('error', __('Failed to update tables. If you didn\'t force an update somehow, please file a bug report.', true));
				return;
		}
	}
}
?>