<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class LibraryController extends AppController {
	public $uses = array('Publisher','Collection','Binding','Author','Artist'
		,'Writer','Colorist','Tag','Album','Series','Image', 'Derivative'
		,'DerivativeType', 'Source', 'Borrower', 'AlbumsBorrower', 'AlbumPrice'
		,'DerivativePrice');
	public $helpers = array();
	public $components = array('ImportExport', 'Installer');
	private $error_log = null;

	public function index() {
		$this->redirect('/');
	}

	public function stats() {
		$this->helpers[] = 'Chart';
		$this->helpers[] = 'Form';
		$this->helpers[] = 'JQuery';
		
		$maxChartResults = Configure::read('Demyo.Stats_years');
		
		// Counts
		$counts['Publisher'] = $this->Publisher->find('count');
		$counts['Collection'] = $this->Collection->find('count');
		$counts['Author'] = $this->Author->find('count');
		$counts['Series'] = $this->Series->find('count');
		$counts['Album'] = $this->Album->find('count', array('conditions' => 'NOT Album.wishlist'));
		$counts['Album_orig'] = $this->Album->find('count', array('conditions' => '(NOT Album.wishlist) AND Album.this_edition = Album.first_edition AND Album.first_edition != \'0000-00-00\' AND Album.first_edition IS NOT NULL'));
		$counts['Album_wish'] = $this->Album->find('count', array('conditions' => 'Album.wishlist'));
		$counts['Album_lended'] = $this->AlbumsBorrower->findLendedCount();
		$counts['Image'] = $this->Image->find('count');
		$counts['Tag'] = $this->Tag->find('count');
		$counts['Derivative'] = $this->Derivative->find('count');
		$this->set('counts', $counts);
		
		// Prices
		$worth['Album'] = $this->AlbumPrice->getWorth();
		$worth['Derivative'] = $this->DerivativePrice->getWorth();
		$this->set('worth', $worth);
		
		$albumTable = $this->Album->tablePrefix . $this->Album->table;
		$charts = array();
		
		// Charts
		$charts['editionsPerYear'] = $this->Album->getEditionsPerYear();
		$charts['purchasePerYear'] = $this->Album->getPurchaseStatisticsByYear();
		
		$this->set('charts', $charts);
		
		// Load logs
		$logs['debug'] = is_readable(LOGS.'debug.log')?file_get_contents(LOGS.'debug.log'):'';
		$logs['error'] = is_readable(LOGS.'error.log')?file_get_contents(LOGS.'error.log'):'';
		$this->set('logs', $logs);
	}
	
	public function search() {
		$this->helpers[] = 'JQuery';
	}
	
	public function dosearch() { // TODO: see if this is still used somewhere
		$this->helpers[] = 'List';
		
		$this->layout = 'ajax';
		
		if (! isset($this->data))
			return;
		if (get_magic_quotes_gpc())
			$input = $this->data['Search']['query'];
		else
			$input = addslashes($this->data['Search']['query']);
		
		$input = str_replace('*', '%', $input);
		$input = str_replace('?', '_', $input);
		if ($input[0] != '%')
			$input = '%'.$input;
		if ($input[strlen($input)-1] != '%')
			$input .= '%';
		
		$results = array();
		
		// Not using an index might look ill-optimised but the fields that are
		// searched are so small it wouldn't give any benefit besides looking
		// more enterprisy
		$modelNames = array('Series', 'Album', 'Author', 'Publisher', 'Collection', 'Source');
		$conditions = array("name LIKE '$input'", "title LIKE '$input'",
			"name LIKE '$input' OR fname LIKE '$input' OR nickname LIKE '$input'", "name LIKE '$input'",
			"name LIKE '$input'", "name LIKE '$input'");
		$fields = array(array('id', 'name'), array('id', 'title'),
			array('id', 'name', 'fname', 'nickname'), array('id', 'name'),
			array('id', 'name'), array('id', 'name'));
		$orders = array('name ASC', 'title ASC', 'name ASC', 'name ASC', 'name ASC', 'name ASC');
		$len = sizeof($modelNames);
		for ($i = 0; $i < $len; $i++)
			if (isset($this->data['Search'][$modelNames[$i]]) && $this->data['Search'][$modelNames[$i]] === 'on')
				$results[$modelNames[$i]] = Set::extract(
					"/{$modelNames[$i]}/.",
					$this->{$modelNames[$i]}->find('all', array('conditions' => $conditions[$i], 'fields' => $fields[$i], 'order' => $orders[$i], 'limit' => null, 'page' => 1, 'recursive' => -1)));
		
		$this->set('results', $results);
	}
	
	private function chartYears(&$resultSet, $maxResults) {
		$notshown = 0;
		$shown = 0;
		$counts = array();
		$labels = array();
		
		// Flatten and restrict
		$i = 0;
		foreach($resultSet as $result) {
			if ($result['year'] == 0) {
				$notshown += $result['count'];
				continue;
			}
			$i++;
			if ($i <= $maxResults) {
				$counts[] = $result['count'];
				$labels[] = $result['year'];
				$shown += $result['count'];
			} else
				$notshown += $result['count'];
		}
		
		return array($counts, $labels, $notshown);
	}
	
	public function export() {
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'JQuery';
		
		if (! isset($this->params['form']['method']))
			return;
		
		$this->layout = 'raw';
		
		$withResources = isset($this->params['form']['withResources']) && $this->params['form']['withResources'] == 'on';
		$onlyWishlist = isset($this->params['form']['onlyWishlist']) && $this->params['form']['onlyWishlist'] == 'on';
		
		if (($exported = $this->ImportExport->export($this->params['form']['method'], $withResources, $onlyWishlist)) === false) {
			$this->cakeError('error', array(array('name' => __('Export failed', true), 'base' => '', 'code' => '',
				'message' => "<ul>\n\t<li>".implode("</li>\n\t<li>", $this->ImportExport->getErrors())."\n</ul>\n")));
			return false;
		}
		
		$this->set('mimetype', $exported['mimetype']);
		$this->set('filename', $exported['filename']);
		if ($withResources) {
			$this->set('zipFile', $exported['zipPath']);
			$this->set('zipName', $exported['zipFileName']);
		} else {
			$this->set('output', $exported['content']);
		}
	}

	public function import() {
		$this->helpers[] = 'HelpBrowser';
		$this->helpers[] = 'JQuery';
		
		if (! isset($this->params['form']['import_file'])) {
			if (isset($this->params['named']['force']) &&
					$this->params['named']['force'] === 'available')
				$this->set('enableForce', true);
			else
				$this->set('enableForce', false);
			return;
		}
		
		try {
			$forceImport = isset($this->params['form']['force_import'])
				&& $this->params['form']['force_import'] === 'on';
			
			if (! $this->ImportExport->import($this->params['form']['method'], $this->params['form']['import_file'], $forceImport)) {
				$this->cakeError('error', array(array('name' => __('Import failed', true), 'base' => '', 'code' => '',
					'message' => $this->ImportExport->getErrors())));
				return false;
			} else
				return $this->flash(__('Import successful', true), '/');
		} catch (SchemaMismatchException $e) {
			return $this->redirect('/library/import/force:available');
		}
	}
	
	public function importStatus() {
		$this->layout = 'raw';
		
		$this->set('status', $this->ImportExport->getImportStatus());
	}
	
	public function orphans() {
		$this->helpers[] = 'Text';
		
		$models = array(
			'Series',
			'Album',
			'Author',
			'Publisher',
			'Collection',
			'Derivative',
			'DerivativeType',
			'Source',
			'Tag',
			'Image',
			'Binding'
		);
		$associations = array(
			array('Album'),
			array('Series'),
			array('Album', 'Derivative'),
			array('Album'),
			array('Album', 'Publisher'),
			array('Series'),
			array('Derivative'),
			array('Derivative'),
			array('Album'),
			array('Album', 'Author', 'Collection', 'Derivative', 'Publisher'),
			array('Album')
		);
		
		$max = count($models);
		$orphanage = array();
		for ($i = 0; $i < $max; $i++) {
			$orphanage[$models[$i]] = $this->{$models[$i]}->find(
				'orphans', array('associations' => $associations[$i]));
		}
		
		// Additional: collections without publisher
		$orphanage['Collection-noPublisher'] = $this->Collection->find(
			'orphans', array('associations' => array('Publisher')));
		
		// Additional: find missing images
		$orphanage['Image-missingFile'] = array();
		foreach ($this->Image->find('all', array('conditions' => 'is_local', 'recursive' => -1)) as $image) {
			if (! file_exists(IMAGES.$image['Image']['url']))
				$orphanage['Image-missingFile'][] = $image;
		}
		
		$this->set('orphanage', $orphanage);
	}

	public function dismissUpdateNotification() {
		Configure::delete('Demyo.lastUpdateInfo');
		$this->Installer->markUpdateCheck(); // Has the side effect of saving the file, effectively deleting above key
	}
}

?>
