<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class PublishersController extends AppController {
	public $scaffold;
	public $uses = array('Publisher');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $components = array('PhpThumb');
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);
	
	public function index() {
		// It's easier to paginate the number of Collections from the
		// Collection model than from here. Else,the different pages would
		// have highly variable lengths
		$this->redirect('/collections');
	}
	
	public function view($id = null) {
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'Feed';
		$this->helpers[] = 'List';
		$this->helpers[] = 'Rte';
		
		if ($id === null)
			$this->redirect('/publishers');

		$this->Publisher->id = $id;
		$this->Publisher->recursive = 1;
		$this->Publisher->contain(array('Image', 'Collection.id', 'Collection.name'));
		if (! $publisher = $this->Publisher->read())
			$this->errorNoSuchEntry(__('This publisher does not exist.', true));
		if (! empty($publisher['Image']['id']))
			$publisher['Image']['thumb_url'] = $this->PhpThumb->getThumbnail($publisher['Image']['url']);
		$publisher['Publisher']['album_count'] = $this->Publisher->Album->getAlbumCount('Publisher', $id);
		
		$this->set('publisher', $publisher);
	}
	
	public function add() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'JQuery';
		$this->helpers[] = 'Rte';
		
		if (!empty($this->data)) {
			if ($this->Publisher->save($this->data)) {
				$this->smartFlash(__('The publisher has been added', true), 'info', array(
					'/publishers/view/'.$this->Publisher->id => __('View publisher', true)
				), false);
			} else
				$this->generateLists();
		}
		
		// else just show the form
		$this->generateLists();
	}
	
	public function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'JQuery';
		$this->helpers[] = 'Rte';
		
		if (empty($this->data)) {
			$this->Publisher->id = $id;
			$this->Publisher->recursive = -1;
			if (! $this->data = $this->Publisher->read())
				$this->errorNoSuchEntry(__('This publisher does not exist.', true));
			$this->generateLists();
		}
		
		else {
			if ($this->Publisher->save($this->data)) {
				$this->smartFlash(__('The publisher has been updated', true), 'info', array(
					'/publishers/view/'.$this->data['Publisher']['id'] => __('View publisher', true)
				));
			} else
				$this->generateLists();
		}
	}
	
	private function generateLists() {
		$this->set('imagesList', $this->Publisher->Image->find('list'));
	}
}

?>
