<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class SeriesController extends AppController {
	public $scaffold;
	public $uses = array('Series');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $components = array('PhpThumb');
	public $paginate = array('limit' => 42, 'page' => 1, 'order'=>array('name' => 'asc'), 'fields' => array('id','name','completed'), 'recursive' => -1);
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);

	public function index() {
		$this->helpers[] = 'MyPaginator';
		
		$this->paginate['limit'] = ($perColumn = Configure::read('Demyo.itemsPerIndexColumns')) * Configure::read('Demyo.columnsPerIndexPage');
		$this->set('perColumn', $perColumn);
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Series->findPageNumberStartsWith('name', $this->params['named']['startsWith'], $this->paginate['limit']);
		}
		
		$this->set('series', $this->paginate('Series'));
	}

	public function view($id = null) {
		$this->helpers[] = 'AlbumTools';
		$this->helpers[] = 'Colorbox';
		$this->helpers[] = 'Form';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'List';
		$this->helpers[] = 'PriceTools';
		$this->helpers[] = 'TagTools';
		$this->helpers[] = 'Rte';
		
		if ($id == null)
			$this->redirect('/series');

		$this->Series->id = $id;
		$this->Series->recursive = 2;
		
		// Fetch only vital information
		$this->Series->contain(array(
			'Album',
			'Album.Publisher' => array('id', 'name'),
			'Album.Collection' => array('id', 'name'),
			'Album.Binding',
			'Album.Artist' => array('id', 'name', 'fname', 'nickname'),
			'Album.Writer' => array('id', 'name', 'fname', 'nickname'),
			'Album.Colorist' => array('id', 'name', 'fname', 'nickname'),
			'Album.AlbumPrice',
			'Album.Tag', 'Album.Borrower',
			'Album.Image' => array('id'), 'Album.Cover',
			'RelatedSeries.id', 'RelatedSeries.name'
		));
		
		if (! $series = $this->Series->read())
			$this->errorNoSuchEntry(__('This series does not exist.', true));
		
		// Look for related editors, make thumbnails, gather tags
		$editors = array();
		$tags = array();
		foreach($series['Album'] as &$album) {
			
			if (! isset($editors[$album['Publisher']['id']]))
				$editors[$album['Publisher']['id']] = $album['Publisher'];
			
			if (! empty($album['Cover']['id']))
				$album['Cover']['thumb_url'] = $this->PhpThumb->getThumbnail($album['Cover']['url']);
			
			foreach ($album['Tag'] as $tag) {
				if (! isset($tags[$tag['id']]))
					$tags[$tag['id']] = $tag;
			}
		}
		$this->set('editors', $editors);
		$this->set('tags', $tags);
		
		// How many do we actually own ?
		$series['owned_albums'] = $this->Series->Album->find('count', array(
			'conditions' => array('series_id' => $id, 'wishlist' => false)
		));
		
		$series['Series']['derivative_count'] = $this->Series->Derivative->getDerivativesCount('Series', $id);
		$this->set('series', $series);
		
		// Look for related authors
		$this->set('artists', $this->getAuthors($series['Album'], 'Artist'));
		$this->set('writers', $this->getAuthors($series['Album'], 'Writer'));
		$this->set('colorists', $this->getAuthors($series['Album'], 'Colorist'));
    }
	
	private function getAuthors($album_list, $author_type) {
		$authors = array();
		foreach($album_list as $album)
			foreach ($album[$author_type] as $author)
				if (! isset($authors[$author['id']]))
					$authors[$author['id']] = $author;
		return $authors;
	}
	
	public function add() {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'Rte';
		
		if (!empty($this->data)) {
			if ($this->Series->save($this->data)) {
				$this->smartFlash(__('The series has been added', true), 'info', array(
					'/series/view/'.$this->Series->id =>  __('View series', true)
				), false);
			} else
				$this->generateLists();
		}
		
		else
			$this->generateLists();
	}
	
	public function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'jQuery';
		$this->helpers[] = 'Rte';
		
		if (empty($this->data)) {
			$this->Series->id = $id;
			$this->Series->contain(array('RelatedSeries.id'));
			if (! $this->data = $this->Series->read())
				$this->errorNoSuchEntry(__('This series does not exist.', true));
			$this->generateLists();
		}
		
		else {
			if ($this->Series->save($this->data)) {
				$this->smartFlash(__('The series has been updated', true), 'info', array(
					'/series/view/'.$this->data['Series']['id'] => __('View series', true)
				));
			} else
				$this->generateLists(); // Failure
		}
	}
	
	public function tag() {
		if (empty($this->data['series_id']) || empty($this->data['tag_name']))
			return $this->redirect('/series');
		
		$this->Series->id = $this->data['series_id'];
		$this->Series->tag(array($this->data['tag_name']));
		
		$this->flash(__('The albums have been tagged', true), '/series/view/'.$this->data['series_id']);
	}

	public function untag() {
		if (empty($this->data['series_id']) || empty($this->data['tag_name']))
			return $this->redirect('/series');
		
		$this->Series->id = $this->data['series_id'];
		$this->Series->unTag(array($this->data['tag_name']));
		
		$this->flash(__('The albums have been untagged', true), '/series/view/'.$this->data['series_id']);
	}
	
	private function generateLists() {
		// Issue #251 Don't allow self-related series
		if ($this->Series->id)
			$conditions = array('conditions' => array('Series.id !=' => $this->Series->id));
		else
			$conditions = array();
		$this->set('seriesList', $this->Series->find('list', $conditions));
	}
}

?>
