<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class TagsController extends AppController {
	public $scaffold;
	public $uses = array('Tag');
	public $helpers = array('Cache', 'Menu', 'Quicksearch');
	public $cacheAction = array('index' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH, 'view' => DEMYO_VIEW_DEFAULT_CACHE_LENGTH);
	
	public function index() {
		$this->helpers[] = 'TagTools';

		$this->set('tags', $this->Tag->getCloud());
	}
	
	public function view($id = null) {
		$this->helpers[] = 'AlbumTools';
		
		if ($id === null)  {
			$this->redirect('/tags/');
			return;
		}
		
		$this->Tag->id = $id;
		$this->Tag->recursive = 2;
		$this->Tag->contain(array(
			'Album.id', 'Album.cycle', 'Album.number', 'Album.number_suffix',
			'Album.title', 'Album.Series.name'
		));
		if (! $tag = $this->Tag->read())
			$this->errorNoSuchEntry(__('This tag does not exist.', true));
		
		// Costy sort but required since the association is too deep to be able
		// to sort from within the database.
		// The number of albums shouldn't be that large anyway (a few hundreds)
		usort($tag['Album'], array('Album', 'sortWithSeries'));
		
		$this->set('tag', $tag);
	}
	
	public function edit($id = null) {
		$this->helpers[] = 'Form';
		$this->helpers[] = 'HelpBrowser';
		
		if (empty($this->data)) {
			$this->Tag->id = $id;
			$this->Tag->recursive = -1;
			if (! $this->data = $this->Tag->read())
				$this->errorNoSuchEntry(__('This tag does not exist.', true));
		}
		
		else {
			if ($this->Tag->save($this->data)) {
				$this->smartFlash(__('The tag has been updated', true), 'info', array(
					'/tags/view/'.$this->data['Tag']['id'] => __('View tag', true)
				));
			}
		}
	}
	
	public function cloud() {
		$this->helpers[] = 'TagTools';
		$this->layout = 'ajax';
		
		if (! isset($this->params['named']['limit']) && ! ctype_digit($this->params['named']['limit']))
			$limit = 5;
		else
			$limit = $this->params['named']['limit'];
		
		$this->set('tags', $this->Tag->getCloud($limit, true, !Configure::read('Demyo.Widget_tagsSorted')));
	}
	
	public function complete() {
		$this->layout = 'raw';
		
		$substring = isset($this->params['url']['q'])?$this->params['url']['q']:'';
		$this->set('tags', $this->Tag->complete($substring));
	}
}

?>
