<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class Image extends AppModel {
	public $name = 'Image';
	public $displayField = 'description';
	public $order = array('description' => 'ASC');
	public $actsAs = array('Containable', 'SimpleXML');
	public $hasMany = array(
		'Album' => array('foreignKey' => 'cover_id'),
		'Author' => array('foreignKey' => 'portrait_id'),
		'Collection' => array('foreignKey' => 'logo_id'),
		'Publisher' => array('foreignKey' => 'logo_id'),
	);
	public $hasAndBelongsToMany = array(
		'Album',
		'Derivative'
	);
	public $validate = array(
		'url' => array('rule' => 'notEmpty'),
		'description' => array('rule' => 'notEmpty')
	);
	
	public function beforeValidate() {
		$this->validate['url']['message'] = __('The path cannot be empty', true);
		$this->validate['description']['message'] = __('The description cannot be empty', true);
	}
	
	public function afterSave($created) {
		// Transitive association from album, not automatically cleared
		clearCache(array('series'));
	}

	function afterFind($results, $primary) {
		if (isset($results['url'])) {
			$this->fixUrl($results);
			return $results;
		}
		
		foreach ($results as &$result) {
			if (isset($result['Image']['url'])) {
				$this->fixUrl($result['Image']);
			}
			else if (isset($result['Cover']['url'])) {
				$this->fixUrl($result['Cover']);
			}
			else if (isset($result['url'])) {
				$this->fixUrl($result);
			}
		}
		return $results;
	}

	/**
	 * Prepends the image URL with the collection images directory, provided it's not already the case.
	 * @param array $img The Image array.
	 */
	function fixUrl(&$img) {
		if (empty($img['is_local'])) {
			debug($img);
		}
		if ($img['is_local'] == 1 && empty($img['is_fixed_url'])) {
			$img['is_fixed_url'] = true;
			$img['url'] = COLLECTION_IMAGES_DIRNAME.DS.$img['url'];
		}
	}
	
	public function getRandomImages($limit = 1) {
		$order = '';
		$db =& ConnectionManager::getDataSource($this->useDbConfig);
		if ($db instanceof DboMysql || $db instanceof DboMysqli)
			$order = 'RAND()';
		else if ($db instanceof DboPostgres || $db instanceof DboSqlite)
			$order = 'RANDOM()';
		else if ($db instanceof DboMssql)
			$order = 'NEWID()';
		
		return $this->find('all', array('limit' => $limit, 'order' => $order, 'recursive' => -1));
	}
}

?>