/*
		Regular schema updates
*/
ALTER TABLE __PREFIX__images MODIFY COLUMN url VARCHAR(255) NOT NULL;
ALTER TABLE __PREFIX__tags ADD COLUMN fgcolor CHAR(7) DEFAULT NULL AFTER name;
ALTER TABLE __PREFIX__tags ADD COLUMN bgcolor CHAR(7) DEFAULT NULL AFTER fgcolor;
ALTER TABLE __PREFIX__albums ADD COLUMN purchase_price FLOAT NULL AFTER purchase_date;
ALTER TABLE __PREFIX__derivatives ADD COLUMN purchase_price FLOAT NULL AFTER purchase_date;
CREATE TABLE __PREFIX__album_prices (
	album_id INT UNSIGNED NOT NULL,
	date DATE NOT NULL,
	price FLOAT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, date),
	CONSTRAINT __PREFIX__fk_album_prices FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;
INSERT INTO __PREFIX__album_prices(album_id, date, price) SELECT id, CURDATE(), price FROM __PREFIX__albums WHERE price > 0;
ALTER TABLE __PREFIX__albums DROP COLUMN price;
CREATE TABLE __PREFIX__derivative_prices (
	derivative_id INT UNSIGNED NOT NULL REFERENCES __PREFIX__derivatives(id),
	date DATE NOT NULL,
	price FLOAT UNSIGNED NOT NULL,
	PRIMARY KEY(derivative_id, date),
	CONSTRAINT __PREFIX__fk_derivative_prices FOREIGN KEY (derivative_id) REFERENCES __PREFIX__derivatives(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;
INSERT INTO __PREFIX__derivative_prices(derivative_id, date, price) SELECT id, CURDATE(), price FROM __PREFIX__derivatives WHERE price > 0;
ALTER TABLE __PREFIX__derivatives DROP COLUMN price;
/*
		Fixing references
*/
/* Fixing references: author to image */
UPDATE __PREFIX__authors a SET portrait_id=null WHERE (SELECT count(*) FROM __PREFIX__images i WHERE i.id = a.portrait_id) = 0;
ALTER TABLE __PREFIX__authors ADD CONSTRAINT __PREFIX__fk_authors_images FOREIGN KEY (portrait_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL;
/* Fixing references: series to series */
DELETE FROM __PREFIX__series_relations WHERE (SELECT count(*) FROM __PREFIX__series s WHERE s.id = __PREFIX__series_relations.main) = 0;
DELETE FROM __PREFIX__series_relations WHERE (SELECT count(*) FROM __PREFIX__series s WHERE s.id = __PREFIX__series_relations.sub) = 0;
ALTER TABLE __PREFIX__series_relations ADD CONSTRAINT __PREFIX__fk_series_relations_main FOREIGN KEY (main) REFERENCES __PREFIX__series(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__series_relations ADD CONSTRAINT __PREFIX__fk_series_relations_sub FOREIGN KEY (sub) REFERENCES __PREFIX__series(id) ON DELETE CASCADE;
/* Fixing references: album to series */
UPDATE __PREFIX__albums a SET series_id=(SELECT id FROM __PREFIX__series s ORDER BY s.name ASC LIMIT 1) WHERE (SELECT count(*) FROM __PREFIX__series s WHERE s.id = a.series_id) = 0;
ALTER TABLE __PREFIX__albums ADD CONSTRAINT __PREFIX__fk_albums_series FOREIGN KEY (series_id) REFERENCES __PREFIX__series(id) ON DELETE RESTRICT;
/* Fixing references: album to publisher */
UPDATE __PREFIX__albums a SET publisher_id=(SELECT id FROM __PREFIX__publishers p ORDER BY p.name ASC LIMIT 1) WHERE (SELECT count(*) FROM __PREFIX__publishers p WHERE p.id = a.publisher_id) = 0;
ALTER TABLE __PREFIX__albums ADD CONSTRAINT __PREFIX__fk_albums_publishers FOREIGN KEY (publisher_id) REFERENCES __PREFIX__publishers(id) ON DELETE RESTRICT;
/* Fixing references: album to collection */
UPDATE __PREFIX__albums a SET collection_id=null WHERE (SELECT count(*) FROM __PREFIX__collections c WHERE c.id = a.collection_id) = 0;
ALTER TABLE __PREFIX__albums ADD CONSTRAINT __PREFIX__fk_albums_collections FOREIGN KEY (collection_id) REFERENCES __PREFIX__collections(id) ON DELETE SET NULL;
/* Fixing references: album to binding */
UPDATE __PREFIX__albums a SET binding_id=null WHERE (SELECT count(*) FROM __PREFIX__bindings b WHERE b.id = a.binding_id) = 0;
ALTER TABLE __PREFIX__albums ADD CONSTRAINT __PREFIX__fk_albums_bindings FOREIGN KEY (binding_id) REFERENCES __PREFIX__bindings(id) ON DELETE SET NULL;
/* Fixing references: album to image (cover) */
UPDATE __PREFIX__albums a SET cover_id=null WHERE (SELECT count(*) FROM __PREFIX__images i WHERE i.id = a.cover_id) = 0;
ALTER TABLE __PREFIX__albums ADD CONSTRAINT __PREFIX__fk_albums_images FOREIGN KEY (cover_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL;
/* Fixing references: album to image (not cover) */
DELETE FROM __PREFIX__albums_images WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = __PREFIX__albums_images.album_id) = 0;
DELETE FROM __PREFIX__albums_images WHERE (SELECT count(*) FROM __PREFIX__images i WHERE i.id = __PREFIX__albums_images.image_id) = 0;
ALTER TABLE __PREFIX__albums_images ADD CONSTRAINT __PREFIX__fk_albums_images_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__albums_images ADD CONSTRAINT __PREFIX__fk_albums_images_image FOREIGN KEY (image_id) REFERENCES __PREFIX__images(id) ON DELETE CASCADE;
/* Fixing references: publisher to image */
UPDATE __PREFIX__publishers p SET logo_id=null WHERE (SELECT count(*) FROM __PREFIX__images i WHERE i.id = p.logo_id) = 0;
ALTER TABLE __PREFIX__publishers ADD CONSTRAINT __PREFIX__fk_publishers_images FOREIGN KEY (logo_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL;
/* Fixing references: collection to image */
UPDATE __PREFIX__collections c SET logo_id=null WHERE (SELECT count(*) FROM __PREFIX__images i WHERE i.id = c.logo_id) = 0;
ALTER TABLE __PREFIX__collections ADD CONSTRAINT __PREFIX__fk_collections_images FOREIGN KEY (logo_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL;
/* Fixing references: collection to publisher */
UPDATE __PREFIX__collections c SET publisher_id=(SELECT id FROM __PREFIX__publishers p ORDER BY p.name ASC LIMIT 1) WHERE (SELECT count(*) FROM __PREFIX__publishers p WHERE p.id = c.publisher_id) = 0;
ALTER TABLE __PREFIX__collections ADD CONSTRAINT __PREFIX__fk_collections_publishers FOREIGN KEY (publisher_id) REFERENCES __PREFIX__publishers(id) ON DELETE RESTRICT;
/* Fixing references: album artists */
DELETE FROM __PREFIX__albums_artists WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = __PREFIX__albums_artists.album_id) = 0;
DELETE FROM __PREFIX__albums_artists WHERE (SELECT count(*) FROM __PREFIX__authors a WHERE a.id = __PREFIX__albums_artists.artist_id) = 0;
ALTER TABLE __PREFIX__albums_artists ADD CONSTRAINT __PREFIX__fk_albums_artists_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__albums_artists ADD CONSTRAINT __PREFIX__fk_albums_artists_author FOREIGN KEY (artist_id) REFERENCES __PREFIX__authors(id) ON DELETE CASCADE;
/* Fixing references: album writers */
DELETE FROM __PREFIX__albums_writers WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = __PREFIX__albums_writers.album_id) = 0;
DELETE FROM __PREFIX__albums_writers WHERE (SELECT count(*) FROM __PREFIX__authors a WHERE a.id = __PREFIX__albums_writers.writer_id) = 0;
ALTER TABLE __PREFIX__albums_writers ADD CONSTRAINT __PREFIX__fk_albums_writers_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__albums_writers ADD CONSTRAINT __PREFIX__fk_albums_writers_author FOREIGN KEY (writer_id) REFERENCES __PREFIX__authors(id) ON DELETE CASCADE;
/* Fixing references: album colorists */
DELETE FROM __PREFIX__albums_colorists WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = __PREFIX__albums_colorists.album_id) = 0;
DELETE FROM __PREFIX__albums_colorists WHERE (SELECT count(*) FROM __PREFIX__authors a WHERE a.id = __PREFIX__albums_colorists.colorist_id) = 0;
ALTER TABLE __PREFIX__albums_colorists ADD CONSTRAINT __PREFIX__fk_albums_colorists_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__albums_colorists ADD CONSTRAINT __PREFIX__fk_albums_colorists_author FOREIGN KEY (colorist_id) REFERENCES __PREFIX__authors(id) ON DELETE CASCADE;
/* Fixing references: album to tags */
DELETE FROM __PREFIX__albums_tags WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = __PREFIX__albums_tags.album_id) = 0;
DELETE FROM __PREFIX__albums_tags WHERE (SELECT count(*) FROM __PREFIX__tags t WHERE t.id = __PREFIX__albums_tags.tag_id) = 0;
ALTER TABLE __PREFIX__albums_tags ADD CONSTRAINT __PREFIX__fk_albums_tags_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__albums_tags ADD CONSTRAINT __PREFIX__fk_albums_tags_tag FOREIGN KEY (tag_id) REFERENCES __PREFIX__tags(id) ON DELETE CASCADE;
/* Fixing references: derivative to series */
UPDATE __PREFIX__derivatives d SET series_id=(SELECT id FROM __PREFIX__series s ORDER BY s.name ASC LIMIT 1) WHERE (SELECT count(*) FROM __PREFIX__series s WHERE s.id = d.series_id) = 0;
ALTER TABLE __PREFIX__derivatives ADD CONSTRAINT __PREFIX__fk_derivatives_series FOREIGN KEY (series_id) REFERENCES __PREFIX__series(id) ON DELETE RESTRICT;
/* Fixing references: derivative to album */
UPDATE __PREFIX__derivatives d SET album_id=null WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = d.album_id) = 0;
ALTER TABLE __PREFIX__derivatives ADD CONSTRAINT __PREFIX__fk_derivatives_albums FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE SET NULL;
/* Fixing references: derivative to author */
UPDATE __PREFIX__derivatives d SET artist_id=null WHERE (SELECT count(*) FROM __PREFIX__authors a WHERE a.id = d.artist_id) = 0;
ALTER TABLE __PREFIX__derivatives ADD CONSTRAINT __PREFIX__fk_derivatives_authors FOREIGN KEY (artist_id) REFERENCES __PREFIX__authors(id) ON DELETE SET NULL;
/* Fixing references: derivative to type */
UPDATE __PREFIX__derivatives d SET derivative_type_id=(SELECT id FROM __PREFIX__derivative_types t ORDER BY t.name ASC LIMIT 1) WHERE (SELECT count(*) FROM __PREFIX__derivative_types t WHERE t.id = d.derivative_type_id) = 0;
ALTER TABLE __PREFIX__derivatives ADD CONSTRAINT __PREFIX__fk_derivatives_types FOREIGN KEY (derivative_type_id) REFERENCES __PREFIX__derivative_types(id) ON DELETE RESTRICT;
/* Fixing references: derivative to source */
UPDATE __PREFIX__derivatives d SET source_id=null WHERE (SELECT count(*) FROM __PREFIX__sources s WHERE s.id = d.source_id) = 0;
ALTER TABLE __PREFIX__derivatives ADD CONSTRAINT __PREFIX__fk_derivatives_sources FOREIGN KEY (source_id) REFERENCES __PREFIX__sources(id) ON DELETE SET NULL;
/* Fixing references: derivative images */
DELETE FROM __PREFIX__derivatives_images WHERE (SELECT count(*) FROM __PREFIX__derivatives d WHERE d.id = __PREFIX__derivatives_images.derivative_id) = 0;
DELETE FROM __PREFIX__derivatives_images WHERE (SELECT count(*) FROM __PREFIX__images i WHERE i.id = __PREFIX__derivatives_images.image_id) = 0;
ALTER TABLE __PREFIX__derivatives_images ADD CONSTRAINT __PREFIX__fk_derivatives_images_derivative FOREIGN KEY (derivative_id) REFERENCES __PREFIX__derivatives(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__derivatives_images ADD CONSTRAINT __PREFIX__fk_derivatives_images_image FOREIGN KEY (image_id) REFERENCES __PREFIX__images(id) ON DELETE CASCADE;
/* Fixing references: album borrowers */
DELETE FROM __PREFIX__albums_borrowers WHERE (SELECT count(*) FROM __PREFIX__albums a WHERE a.id = __PREFIX__albums_borrowers.album_id) = 0;
DELETE FROM __PREFIX__albums_borrowers WHERE (SELECT count(*) FROM __PREFIX__borrowers b WHERE b.id = __PREFIX__albums_borrowers.borrower_id) = 0;
ALTER TABLE __PREFIX__albums_borrowers ADD CONSTRAINT __PREFIX__fk_albums_borrowers_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE;
ALTER TABLE __PREFIX__albums_borrowers ADD CONSTRAINT __PREFIX__fk_albums_borrowers_borrower FOREIGN KEY (borrower_id) REFERENCES __PREFIX__borrowers(id) ON DELETE CASCADE;