DROP TABLE IF EXISTS __PREFIX__searches;
DROP TABLE IF EXISTS __PREFIX__albums_borrowers;
DROP TABLE IF EXISTS __PREFIX__borrowers;
DROP TABLE IF EXISTS __PREFIX__derivatives_images;
DROP TABLE IF EXISTS __PREFIX__derivative_prices;
DROP TABLE IF EXISTS __PREFIX__derivatives;
DROP TABLE IF EXISTS __PREFIX__derivative_types;
DROP TABLE IF EXISTS __PREFIX__sources;
DROP TABLE IF EXISTS __PREFIX__albums_tags;
DROP TABLE IF EXISTS __PREFIX__tags;
DROP TABLE IF EXISTS __PREFIX__albums_colorists;
DROP TABLE IF EXISTS __PREFIX__albums_writers;
DROP TABLE IF EXISTS __PREFIX__albums_artists;
DROP TABLE IF EXISTS __PREFIX__albums_images;
DROP TABLE IF EXISTS __PREFIX__album_prices;
DROP TABLE IF EXISTS __PREFIX__albums;
DROP TABLE IF EXISTS __PREFIX__bindings;
DROP TABLE IF EXISTS __PREFIX__series_relations;
DROP TABLE IF EXISTS __PREFIX__series;
DROP TABLE IF EXISTS __PREFIX__authors;
DROP TABLE IF EXISTS __PREFIX__collections;
DROP TABLE IF EXISTS __PREFIX__publishers;
DROP TABLE IF EXISTS __PREFIX__images;

CREATE TABLE __PREFIX__images (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	url VARCHAR(255) NOT NULL,
	is_local TINYINT(1) DEFAULT '1',
	description TEXT NULL,
	INDEX(description(20))
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__publishers (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	website VARCHAR(255) NULL,
	feed VARCHAR(255) NULL,
	history TEXT NULL,
	logo_id INT UNSIGNED NULL,
	INDEX(name),
	CONSTRAINT __PREFIX__fk_publishers_images FOREIGN KEY (logo_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__collections (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	website VARCHAR(255) NULL,
	feed VARCHAR(255) NULL,
	history TEXT NULL,
	logo_id INT UNSIGNED NULL,
	publisher_id INT UNSIGNED NOT NULL,
	INDEX(name),
	CONSTRAINT __PREFIX__fk_collections_images FOREIGN KEY (logo_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL,
	CONSTRAINT __PREFIX__fk_collections_publishers FOREIGN KEY (publisher_id) REFERENCES __PREFIX__publishers(id) ON DELETE RESTRICT
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__authors (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL,
	fname VARCHAR(127) NULL,
	nickname VARCHAR(127) NULL,
	biography TEXT NULL,
	portrait_id INT UNSIGNED NULL,
	website VARCHAR(255) NULL,
	INDEX(name),
	CONSTRAINT __PREFIX__fk_authors_images FOREIGN KEY (portrait_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__series (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	summary TEXT NULL,
	completed TINYINT(1) DEFAULT '0',
	comment TEXT NULL,
	website VARCHAR(255) NULL,
	INDEX(name)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__series_relations (
	main INT UNSIGNED,
	sub INT UNSIGNED,
	PRIMARY KEY(main, sub),
	INDEX(main),
	CONSTRAINT __PREFIX__fk_series_relations_main FOREIGN KEY (main) REFERENCES __PREFIX__series(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_series_relations_sub FOREIGN KEY (sub) REFERENCES __PREFIX__series(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__bindings (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__albums (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	series_id INT UNSIGNED NOT NULL,
	cycle SMALLINT UNSIGNED NULL,
	number FLOAT UNSIGNED NULL,
	number_suffix VARCHAR(5) NULL,
	title VARCHAR(255) NULL, /* One shots sometimes have no title */
	publisher_id INT UNSIGNED NOT NULL,
	collection_id INT UNSIGNED NULL,
	first_edition DATE NULL,
	this_edition DATE NULL,
	isbn VARCHAR(18) NULL, /* ISBN-10 or ISBN-13 with at most 5 separators (spaces, dashes, ...) */
	purchase_date DATE NULL,
	purchase_price FLOAT NULL,
	wishlist TINYINT(1) DEFAULT '0',
	binding_id INT UNSIGNED NULL,
	cover_id INT UNSIGNED NULL,
	summary TEXT NULL,
	comment TEXT NULL,
	height FLOAT UNSIGNED NULL,
	width FLOAT UNSIGNED NULL,
	pages INT UNSIGNED NULL,
	INDEX(title),
	CONSTRAINT __PREFIX__fk_albums_series FOREIGN KEY (series_id) REFERENCES __PREFIX__series(id) ON DELETE RESTRICT,
	CONSTRAINT __PREFIX__fk_albums_publishers FOREIGN KEY (publisher_id) REFERENCES __PREFIX__publishers(id) ON DELETE RESTRICT,
	CONSTRAINT __PREFIX__fk_albums_collections FOREIGN KEY (collection_id) REFERENCES __PREFIX__collections(id) ON DELETE SET NULL,
	CONSTRAINT __PREFIX__fk_albums_bindings FOREIGN KEY (binding_id) REFERENCES __PREFIX__bindings(id) ON DELETE SET NULL,
	CONSTRAINT __PREFIX__fk_albums_images FOREIGN KEY (cover_id) REFERENCES __PREFIX__images(id) ON DELETE SET NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__album_prices (
	album_id INT UNSIGNED NOT NULL,
	date DATE NOT NULL,
	price FLOAT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, date),
	CONSTRAINT __PREFIX__fk_album_prices FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__albums_images (
	album_id INT UNSIGNED NOT NULL,
	image_id INT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, image_id),
	CONSTRAINT __PREFIX__fk_albums_images_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_albums_images_image FOREIGN KEY (image_id) REFERENCES __PREFIX__images(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__albums_artists (
	album_id INT UNSIGNED NOT NULL,
	artist_id INT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, artist_id),
	CONSTRAINT __PREFIX__fk_albums_artists_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_albums_artists_author FOREIGN KEY (artist_id) REFERENCES __PREFIX__authors(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;
CREATE TABLE __PREFIX__albums_writers (
	album_id INT UNSIGNED NOT NULL,
	writer_id INT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, writer_id),
	CONSTRAINT __PREFIX__fk_albums_writers_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_albums_writers_author FOREIGN KEY (writer_id) REFERENCES __PREFIX__authors(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;
CREATE TABLE __PREFIX__albums_colorists (
	album_id INT UNSIGNED NOT NULL,
	colorist_id INT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, colorist_id),
	CONSTRAINT __PREFIX__fk_albums_colorists_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_albums_colorists_author FOREIGN KEY (colorist_id) REFERENCES __PREFIX__authors(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__tags (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL UNIQUE,
	fgcolor CHAR(7) DEFAULT NULL,
	bgcolor CHAR(7) DEFAULT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__albums_tags (
	album_id INT UNSIGNED NOT NULL,
	tag_id INT UNSIGNED NOT NULL,
	PRIMARY KEY(album_id, tag_id),
	CONSTRAINT __PREFIX__fk_albums_tags_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_albums_tags_tag FOREIGN KEY (tag_id) REFERENCES __PREFIX__tags(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__sources (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL,
	owner VARCHAR(127) NULL,
	email VARCHAR(255) NULL,
	website VARCHAR(255) NULL,
	address VARCHAR(255) NULL,
	phone_number VARCHAR(31) NULL,
	history TEXT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__derivative_types (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__derivatives (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	series_id INT UNSIGNED NOT NULL,
	album_id INT UNSIGNED NULL,
	artist_id INT UNSIGNED NULL,
	derivative_type_id INT UNSIGNED NOT NULL,
	colors SMALLINT UNSIGNED NULL,
	/* Source could be an editor, a library, the author himself. */
	source_id INT UNSIGNED NULL,
	number SMALLINT UNSIGNED NULL,
	total SMALLINT UNSIGNED NULL, 
	signed TINYINT(1) DEFAULT '1',
	authors_copy TINYINT(1) DEFAULT '0',
	restricted_sale TINYINT(1) DEFAULT '0',
	description TEXT NULL,
	width float UNSIGNED NULL,
	height float UNSIGNED NULL,
	depth float UNSIGNED NULL,
	purchase_date DATE NULL,
	purchase_price FLOAT NULL,
	price FLOAT UNSIGNED NULL,
	CONSTRAINT __PREFIX__fk_derivatives_series FOREIGN KEY (series_id) REFERENCES __PREFIX__series(id) ON DELETE RESTRICT,
	CONSTRAINT __PREFIX__fk_derivatives_albums FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE SET NULL,
	CONSTRAINT __PREFIX__fk_derivatives_authors FOREIGN KEY (artist_id) REFERENCES __PREFIX__authors(id) ON DELETE SET NULL,
	CONSTRAINT __PREFIX__fk_derivatives_types FOREIGN KEY (derivative_type_id) REFERENCES __PREFIX__derivative_types(id) ON DELETE RESTRICT,
	CONSTRAINT __PREFIX__fk_derivatives_sources FOREIGN KEY (source_id) REFERENCES __PREFIX__sources(id) ON DELETE SET NULL
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__derivative_prices (
	derivative_id INT UNSIGNED NOT NULL,
	date DATE NOT NULL,
	price FLOAT UNSIGNED NOT NULL,
	PRIMARY KEY(derivative_id, date),
	CONSTRAINT __PREFIX__fk_derivative_prices FOREIGN KEY (derivative_id) REFERENCES __PREFIX__derivatives(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__derivatives_images (
	derivative_id INT UNSIGNED NOT NULL,
	image_id INT UNSIGNED NOT NULL,
	PRIMARY KEY(derivative_id, image_id),
	CONSTRAINT __PREFIX__fk_derivatives_images_derivative FOREIGN KEY (derivative_id) REFERENCES __PREFIX__derivatives(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_derivatives_images_image FOREIGN KEY (image_id) REFERENCES __PREFIX__images(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__borrowers (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(127) NOT NULL,
	fname VARCHAR(127) NULL,
	email VARCHAR(255) NULL,
	UNIQUE(name, fname)
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__albums_borrowers (
	album_id INT UNSIGNED NOT NULL,
	borrower_id INT UNSIGNED NOT NULL,
	borrow_date DATE NOT NULL,
	return_date DATE NULL,
	PRIMARY KEY(album_id, borrower_id, borrow_date),
	CONSTRAINT __PREFIX__fk_albums_borrowers_album FOREIGN KEY (album_id) REFERENCES __PREFIX__albums(id) ON DELETE CASCADE,
	CONSTRAINT __PREFIX__fk_albums_borrowers_borrower FOREIGN KEY (borrower_id) REFERENCES __PREFIX__borrowers(id) ON DELETE CASCADE
) engine=InnoDB CHARACTER SET utf8;

CREATE TABLE __PREFIX__searches (
	id INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
	model VARCHAR(255) NOT NULL, /* The kind of item being searched. */
	status VARCHAR(63) NOT NULL, /* The status. Either temporary or saved. */
	name VARCHAR(255) NULL, /* User-specified name */
	criteria VARCHAR(4094) NOT NULL, /* The criteria for the search */
	parameters VARCHAR(4094) NULL, /* Some addition parameters for the search (e.g. how to display the result for albums) */
	last_used DATETIME NOT NULL, /* Last date of usage, for automated pruning */
	UNIQUE(model, status, name),
	INDEX(model)
) engine=InnoDB CHARACTER SET utf8;

SELECT 'done';
