<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class Tag extends AppModel {
	public $name = 'Tag';
	public $actsAs = array('Completable', 'Containable', 'SimpleXML');
	public $hasAndBelongsToMany = array(
		'Album' =>
			array('className' => 'Album',
				'uniq' => true,
			)
	);
	public $validate = array(
		'name' => array('rule' => 'notEmpty'),
		'fgcolor' => array('rule' => array('hex_color', false), 'allowEmpty' => true),
		'bgcolor' => array('rule' => array('hex_color', false), 'allowEmpty' => true)
	);
	
	public function beforeValidate() {
		$this->validate['name']['message'] = __('The name cannot be empty', true);
		$this->validate['fgcolor']['message'] = __('This is not a valid color', true);
		$this->validate['bgcolor']['message'] = __('This is not a valid color', true);
	}
	
	public function getCloud($limit = -1, $randomPick = false, $randomOrder = false, $logBase = 1.1) {
		if ($limit === -1)
			$limit = '';
		else
			$limit = "LIMIT $limit";
		
		$db =& ConnectionManager::getDataSource($this->useDbConfig);
		if ($randomPick && ($db instanceof DboMysql || $db instanceof DboMysqli))
			$order = 'RAND()';
		else if ($randomPick && ($db instanceof DboPostgres || $db instanceof DboSqlite))
			$order = 'RANDOM()';
		else if ($randomPick && ($db instanceof DboMssql))
			$order = 'NEWID()';
		else
			$order = 'name ASC';
			
		 $cloud = $this->query("SELECT id, name, fgcolor, bgcolor, COUNT(album_id) AS occurences FROM {$this->tablePrefix}{$this->table} Tag"
			." LEFT JOIN {$this->tablePrefix}{$this->hasAndBelongsToMany['Album']['joinTable']} AlbumTags ON Tag.id=AlbumTags.tag_id"
			." GROUP BY tag_id ORDER BY $order $limit");

		// Clean results and normalize them
		foreach ($cloud as &$drop) {
			$drop['Tag']['occurences_normalized'] = log($drop[0]['occurences']+1, $logBase);
			$drop['Tag']['occurences'] = $drop[0]['occurences'];
			unset($drop[0]);
		}
		
		if ($randomPick && !$randomOrder)
			usort($cloud, array($this, 'cmpCloud'));
		if (!$randomPick && $randomOrder)
			shuffle($cloud);
		
		return $cloud;
	}
	
	private function cmpCloud($a, $b) {
		return strcasecmp($a['Tag']['name'], $b['Tag']['name']);
	}
	
	public function getFromCSV($csv) {
		return $this->getFromArray(preg_split("/\s*,\s*/", $csv));
	}
	
	public function getFromArray($names) {
		$ids = array();
		
		foreach($names as $name) {
			if ($name == '') // Skip duplicate commas
				continue;
			
			$id = $this->find(array('name' => $name), 'id', null, -1);
			if (! empty($id)) {
				array_push($ids, $id['Tag']['id']);
			} else {
				$this->create();
				if ($this->save(array('Tag' => array('name' => $name))))
					array_push($ids, $this->getInsertID());
			}
		}
		
		return $ids;
	}
	
	public function afterSave($created) {
		// Transitive association from album, not automatically cleared
		clearCache(array('series'));
	}
}

?>
