<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/** Provides helper methods for the Colorbox library.
 * http://colorpowered.com/colorbox/
 */
class ColorboxHelper extends AppHelper {
	public $helpers = array('Html', 'jQuery');
	
	private $enableColorbox = false;
	
	public function afterRender() {
		if (! $this->enableColorbox)
			return;
		
		$code = <<<EOS
jQuery(function() {
	jQuery("a[rel^='colorbox-gallery']").colorbox({
		maxWidth: '90%',
		maxHeight: '90%',
		photo: true,
		opacity: 0.75,
		current: "{current} / {total}"
	});
});
EOS;
		echo $this->Html->scriptBlock($code);
	}

	public function thumb($imageUrl, $thumbUrl, $options = null) {
		if (isset($options['group']) && $options['group'] != null)
			$rel = "rel=\"colorbox-gallery-{$options['group']}\"";
		else
			$rel = "rel=\"colorbox-gallery\"";
		
		if (empty($imageUrl)) {
			$imageUrl = '#';
			$rel = '';
		} else if (!isset($options['isLocal']) || $options['isLocal'])
			$imageUrl = $this->url('/'.IMAGES_URL.$imageUrl);
		
		if ($rel != '' && !$this->enableColorbox) {
			$this->enableColorbox = true;
			$this->jQuery->includeLib('colorbox');
		}

		$title = isset($options['title'])?'title="'.h($options['title']).'"':'';
		$options['alt'] = isset($options['alt'])?$options['alt']:__('Thumbnail', true);
		$options['class'] = isset($options['class'])?$options['class']:'thumb';
		
		$legend = isset($options['legend'])?$options['legend']:'';
		
		$divID = isset($options['div-id'])?"id=\"{$options['div-id']}\"":'';
		unset($options['div-id']);
		
		unset($options['group'], $options['isLocal'], $options['legend']);
		
		if (empty($thumbUrl))
			$thumbUrl = 'default-thumbnail.png';
		return "<div class=\"thumb\" $divID><a href=\"$imageUrl\" $rel $title>".$this->Html->image($thumbUrl, $options)."</a><div class=\"legend\">$legend</div></div>\n";
	}

	/** Returns HTML table content (excluding opening and closing table tags)
	 * for a gallery of images.
	 *
	 * @param array $images A list of images to show, each element being a hash
	 * containing at least 'url', 'thumb_url' and 'description'. 'legend' can
	 * also be supplied.
	 * @param string $galleryName The name for gallery elements, random by
	 * default
	 * @param integer $perLine The number of items per line, defaults to
	 * the setting in the configuration.
	 * @return string
	 */
	public function gallery($images, $galleryName = null, $perLine = 0) {
		$total = count($images);
		if ($total < 1)
			return;
		
		if ($galleryName === null)
			$galleryName = 'gal'.rand(0, 10000);
		if ($perLine === 0)
			$perLine = Configure::read('Demyo.Gallery_thumbsPerLine');
		
		$gallery='<tr>';
		$i = 0;
		foreach($images as $image) {
			$gallery .= '<td>'.$this->thumb($image['url'], $image['thumb_url'],
					array('title' => $image['description'],
						'group' => $galleryName,
						'legend' => isset($image['legend'])?$image['legend']:''
					)).'</td>';
			if ((++$i % $perLine) === 0 && ($i !== $total))
				$gallery.='</tr><tr>';
		}
		$gallery.='</tr>';
		
		return $gallery;
	}
}
?>