<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class JQueryHelper extends AppHelper {
	public $helpers = array('Html');

	/* Structure:
	 * 'basename' => array('file' => 'actual_filename', 'depends' [optional] => $basename_or_array_of_basenames, 'included' => false)
	 */
	private $inclusions = array(
		'.base-real' => array('file' => 'jquery', 'included' => false),
		'base' => array('file' => 'jquery.noconflict', 'depends' => '.base-real', 'included' => false),
		'json' => array('file' => 'json2.js', 'included' => false),
		'ui' => array('file' => 'jquery.ui', 'depends' => 'base', 'included' => false),
		'color' => array('file' => 'jquery.color', 'depends' => 'base', 'included' => false),
		'slider' => array('file' => 'jquery.slider', 'depends' => array('base', 'delayed_event', 'dimensions'), 'included' => false),
		'delayed_event' => array('file' => 'jquery.delayedevent', 'depends' => 'base', 'included' => false),
		'dimensions' => array('file' => 'jquery.dimensions', 'depends' => 'base', 'included' => false),
		'datepicker' => array('file' => 'jquery.datepicker', 'css' => 'jquery.datepicker', 'depends' => 'base', 'included' => false),
		'number_field' => array('file' => 'jquery.numberfield', 'depends' => array('base', 'dimensions'), 'included' => false),
		'tag_field' => array('file' => 'jquery.tagfield', 'depends' => array('base', 'autocomplete'), 'included' => false),
		'hotkeys' => array('file' => 'jquery.hotkeys', 'depends' => 'base', 'included' => false),
		'autocomplete' => array('file' => 'jquery.autocomplete', 'depends' => 'base', 'included' => false),
		'colorbox' => array('file' => 'jquery.colorbox', 'css' => 'colorbox', 'depends' => 'base', 'included' => false),
		'colorpicker' => array('file' => 'farbtastic', 'depends' => 'base', 'included' => false),
		'file_browser' => array('file' => 'jquery.filebrowser', 'depends' => array('json', 'colorbox'), 'included' => false),
		'advanced_search' => array('file' => 'jquery.advancedsearch', 'depends' => array('datepicker', 'number_field'), 'included' => false),
		// Was optional for autocomplete in case server didn't answer before
		// next input character: jquery.ajaxQueue
	);

	/** Includes a library
	 * @param $name The library to include. The name should be referenced
	 * in this class' inclusions map. If a library depends on others, the
	 * dependencies will be loaded first (and so will theirs, recursively).
	 * An already included library is not included a second time.
	 * If this argument is an array, all in the array will be included.
	 * @return nothing
	 */
	public function includeLib($name) {
		if (is_array($name)) {
			foreach ($name as $item)
				$this->includeLib($item);
			return;
		}
		
		if (! isset($this->inclusions[$name])) {
			trigger_error("$name is not a recognised Javascript library");
			return false;
		}
		
		$inclusion = &$this->inclusions[$name];
		if ($inclusion['included'])
			return true; // Already included, skip
		
		if (isset($inclusion['depends']))
			$this->includeLib($inclusion['depends']);
		
		if (isset($inclusion['css'])) {
			$stylesheets = $inclusion['css'];
			if (! is_array($stylesheets))
				$stylesheets = array($stylesheets);
			foreach ($stylesheets as $css)
				$this->Html->css($css, null, array('inline' => false));
		}
		
		$this->Html->script($inclusion['file'], false);
		$inclusion['included'] = true;
		return true;
	}

	/** Outputs a flashing warning */
	public function warning($text, $echo = true) {
		$this->includeLib('color');

		$warn = <<<EOS
<div class="warning">$text</div>
<script type="text/javascript">
jQuery(document).ready(function() {
	jQuery('div.warning').css('background-color', '#f55');
	jQuery('div.warning').animate({backgroundColor: '#feb'}, 2000);
});
</script>

EOS;
		if ($echo)
			echo $warn;
		else
			return $warn;
	}
	
	private $shortcuts = '';
	public function afterRender() {
		if ($this->shortcuts == '')
			return;
		
		echo $this->Html->scriptBlock($this->shortcuts);
	}
	public function shortcutGoto($key, $url) {
		$this->includeLib('hotkeys');
		
		$code = "jQuery(document).bind('keydown', { combi: '$key', disableInInput: true }, function() {\n";
		
		if ($key == 'left' || $key == 'right') {
			$code .= <<<EOS
	// Check if a colorbox is currently active. If so, Ignore the goto
	var box = jQuery('div#colorbox');
	if (box.size() > 0 && box.css('display') != 'none')
		return;
EOS;
		}
		
		$code .= "\n\twindow.location = '$url';\n});\n";
		
		$this->shortcuts .= $code;
	}
	
	public function clearValidationErrorsOnChange() {
		$this->includeLib('base');
		$code = <<<EOS
jQuery(function() {
		jQuery('div.error-message').each(function() {
			// Add error class when not already the case
			var td = jQuery(this).parents('td').get(0);
			jQuery('input, select, textarea', td).addClass('form-error');
		});
		jQuery('.form-error').change(function() {
			self = jQuery(this);
			self.removeClass('form-error');
			jQuery('div.error-message', self.parents('td').get(0)).remove();
		});
});
EOS;
		return $this->Html->scriptBlock($code);
	}
}
?>
