<?php
/* Copyright 2007-2011 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

set_include_path(get_include_path() . PATH_SEPARATOR . APP.DS.'vendors'.DS.'Pear');

/**
 * Helper class for Rich Text Editing in Demyo.
 */
class RteHelper extends AppHelper {
	public $helpers = array('Html', 'Form');

	private $enableRte = false;

	/**
	 * Shows a Rich Text textarea. 
	 * @param string $fieldName The field name
	 * @param array $options The options. All but the class are passed to Form::textArea
	 */
	public function textarea($fieldName, $options = array()) {
		$this->enableRte = true;
		$options['class'] = 'richtext';

		if (!isset($options['rows'])) {
			$options['rows'] = 5;
		}
		if (!isset($options['cols'])) {
			$options['cols'] = 5;
		}

		return $this->Form->textarea($fieldName, $options);
	}

	public function parse($content) {
		$content = preg_replace_callback('/href="((Album|Author|Binding|Collection|Derivative|DerivativeType|Publisher|Series|Source):([0-9]+))"/',
			array($this, 'internalLink'), $content);
		// Run substitutions on non-html content only
		$content = preg_replace_callback('/>([^<]+)</', array($this, 'matchOnContent'), $content);
		//$content = $this->localeBasedSubstitutions($content);
		
		return $content;
	}

	private function internalLink($matches) {
		$target = $matches[2];

		$target = Inflector::tableize($target);
		$id = $matches[3];

		return 'href="'.Router::url("/$target/view/$id").'"';
	}

	private function matchOnContent($matches) {
		$content = $matches[1];
		$content = $this->localeBasedSubstitutions($content);
		return ">$content<";
	}

	private function localeBasedSubstitutions($content) {
		$substitutions = Configure::read('Demyo-l10n.richTextContentSubstitutions');
		foreach ($substitutions as $pattern => $replacement) {
			$content = preg_replace($pattern, $replacement, $content);
		}
		return $content;
	}

	function afterRender() {
		if ($this->enableRte) {
			$this->Html->script('demyo_tmce_helper.js', false);
			$this->Html->script('tiny_mce/tiny_mce_gzip.js', false);
			$this->Html->script('tiny_mce/tiny_mce.js', false);

			$locale = Configure::read('Demyo-l10n.tinyMceLocale');

			$code = <<<EOS
tinyMCE_GZ.init({
	plugins: 'nonbreaking,preview,tabfocus,visualchars,xhtmlxtras',
	theme: 'advanced',
	language: '$locale',
	disk_cache: true,
	debug: false
});
EOS;
			echo $this->Html->scriptBlock($code);
			$code = <<<EOS
tinyMCE.init({
	plugins : 'nonbreaking,preview,tabfocus,visualchars,xhtmlxtras',
	theme: 'advanced',
	theme_advanced_toolbar_location: 'top',
	theme_advanced_toolbar_align: 'left',
	theme_advanced_statusbar_location: 'bottom',
	theme_advanced_resizing: true,
	theme_advanced_buttons1: 'formatselect,|,bold,italic,underline,del,|,sub,sup,|,abbr,acronym,|,forecolor,backcolor',
	theme_advanced_buttons2: 'bullist,numlist,|,justifyleft,justifycenter,justifyright,justifyfull,|,outdent,indent,blockquote,|,link,unlink,|,charmap,|,visualchars,nonbreaking,|,cleanup,removeformat,code',
	theme_advanced_buttons3: '',
	theme_advanced_buttons4: '',
	theme_advanced_resizing_use_cookie : false,
	language: '$locale',
	debug: false,
	entity_encoding : 'raw',
	mode: 'specific_textareas',
	editor_selector: 'richtext'
});
EOS;
			echo $this->Html->scriptBlock($code);
		}
	}
}
?>