/* Copyright 2007-2011 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/*
jquery.filebrowser

A heavily-bound to Demyo AJAX file browser.
 */
(function() {
	jQuery.extend({
		fileBrowserRegexes: {
			images: /\.(gif|png|bmp|jpg|jpeg)$/i,
			none: /^$/
		}
	})
})();

(function() {
	jQuery.fn.extend({
		fileBrowser: function(settings) {
			// setup configuration
			settings = jQuery.extend({
				input: '#path',
				selectFolders: false,
				selectablePattern: /^.*$/i,
				showHidden: false,
				basePath: '/',
				previewBaseUrl: null
				// Also expecting:
				//	url: the path to the browser URL
				//  locale: the locale settings. See jQuery.FileBrowserLocales
			}, settings);

			// Strip trailing slash from preview URL if the base starts with one
			if (settings.previewBaseUrl != null
					&& settings.basePath.match(/^\//)
					&& settings.previewBaseUrl.match(/\/$/))
				settings.previewBaseUrl = settings.previewBaseUrl.substr(0, settings.previewBaseUrl.length-1);
			
			return this.each(setUpField);
			
			function handleError(listElem, message) {
				listElem.removeClass('loading');
				listElem.append('<span class="error">Error: ' + message + '</span>');
			}

			function dirLoadHandler() {
				var jqThis = jQuery(this);
				var listItem = jqThis.parent();
				listItem.addClass('open');
				loadContent(listItem, listItem.parent().attr('fullPath') + listItem.attr('localPath') + '/');
				jqThis.unbind('click');
				jqThis.attr('title', settings.locale.textCloseFolderHint);
				jqThis.click(function() {
					var _jqThis = jQuery(this);
					var _listItem = _jqThis.parent()
					jQuery('> ul', _listItem).remove();
					_listItem.removeClass('open');
					_jqThis.unbind('click');
					_jqThis.click(dirLoadHandler);
					_jqThis.attr('title', settings.locale.textOpenFolderHint);
				});
			}
			
			function selectItem() {
				var localListItem = jQuery(this).parent();
				jQuery(settings.input).val(
					localListItem.parent().attr('fullPath') + localListItem.attr('localPath')
				);
				// The following does not persist when closing and reopening a folder and is confusing
				//jQuery('.selected', settings.topMost).removeClass('selected');
				//localListItem.addClass('selected');
			}

			function previewImage() {
				var localListItem = jQuery(this).parent();
				jQuery.fn.colorbox({
					href: settings.previewBaseUrl + localListItem.parent().attr('fullPath') + localListItem.attr('localPath'),
					maxWidth: '90%',
					maxHeight: '90%',
					photo: true,
					opacity: 0.75,
					open: true
				});
			}
			
			function fillList(target, list) {
				target.empty();

				// Directories
				for (i = 0; i < list.dirs.length; i++) {
					listItem = jQuery('<li></li>');
					listItem.attr('localPath', list.dirs[i]);
					span = jQuery('<span>'+list.dirs[i]+'</span>');
					span.click(dirLoadHandler);
					span.attr('title', settings.locale.textOpenFolderHint);
					listItem.addClass('folder');
					listItem.append(span);
					if (settings.selectFolders) {
						select = jQuery('<span class="notice">'+settings.locale.textSelectFolder+'</span>');
						select.click(selectItem);
						listItem.append(select);
					}
					target.append(listItem);
				}

				// Files
				for (i = 0; i < list.files.length; i++) {
					listItem = jQuery('<li></li>');
					listItem.attr('localPath', list.files[i]);
					span = jQuery('<span>'+list.files[i]+'</span>');
					span.click(selectItem);
					span.attr('title', settings.locale.textSelectHint);
					listItem.append(span);
					// For images, add a specific class and a preview link
					if (list.files[i].match(jQuery.fileBrowserRegexes.images)) {
						listItem.addClass('image');
						// Add preview link
						if (settings.previewBaseUrl != null) {
							preview = jQuery('<span class="notice">'+settings.locale.textPreview+'</span>');
							preview.click(previewImage);
							listItem.append(preview);
						}
					}
					target.append(listItem);
				}

				if (list.dirs.length <= 0 && list.files.length <= 0)
					target.html('<li><span class="notice empty">'+settings.locale.textEmptyFolder+'</span></li>')
			}

			function loadContent(container, path) {
				// Create an inner list
				var rand = Math.floor(Math.random()*100000);
				container.append('<ul id="jqfb-'+rand+'"><li>&nbsp;</li></ul>');
				target = jQuery('ul#jqfb-'+rand, container);
				target.attr('fullPath', path);

				// Mark the list as being loaded
				listElem = jQuery ('> li', target);
				listElem.addClass('loading');

				// Trigger loading
				jQuery.ajax({
						type: 'GET',
						url: settings.url,
						//dataType: 'json',
						data: {
							selectable: settings.selectablePattern.toString(),
							showHidden: settings.showHidden,
							path: path
						},
						success: function(data, status) {
							var fileList;
							try {
								fileList = JSON.parse(data);
							} catch (e) {
								handleError(listElem, e);
							}
							if (fileList.error !== undefined)
								handleError(listElem, fileList.error);
							else
								fillList(target, fileList);
						},
						error: function(req, status, cause) {
							handleError(listElem, status);
						},
						cache: false
				});
			}

			function setUpField() {
				settings.topMost = this;
				loadContent(jQuery(this), settings.basePath);
			}
		}
	})
})();

/* Locales. */
jQuery.extend({
	FileBrowserLocales: {
		eng: {
			textEmptyFolder: 'This folder is empty',
			textOpenFolderHint: 'Click to open this folder',
			textCloseFolderHint: 'Click to close this folder',
			textSelectHint: 'Click to select',
			textSelectFolder: '(Select)',
			textPreview: '(Preview)',
		},
		
		fre: {
			textEmptyFolder: 'Ce dossier est vide',
			textOpenFolderHint: 'Cliquez pour ouvrir ce dossier',
			textCloseFolderHint: 'Cliquez pour fermer ce dossier',
			textSelectHint: 'Cliquez pour sélectionner',
			textSelectFolder: '(Sélectionner)',
			textPreview: '(Prévisualiser)',
		}
	}
});