/* Copyright 2007-2010 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/* Register an input field as a number field.
 * This plugins assumes that your input field(s) has an id
 *
 * Configurable settings:
 *		min: the minimal value (def 0)
 *		max: the maximal value (def 0xFFFF)
 *		defaultNumber: the default value (def '')
 *		increment: the step size to increment or decrement
 * 		allowEmpty: true if the field can be empty (def true)
 * 		allowFloat: true if float number are allowed (def false)
 * 		upText: the text to show as 'up arrow' (def '&uarr;')
 * 		downText: the text to show as 'down arrow' (def '&darr;')
 *
 * Styleable classes: div.numberHelper, .numberHelper span.nhUp, .numberHelper span.nhDown
 * 
 * Depends on jquery.dimensions
 */
(function() {
jQuery.fn.extend({
	numberField: function(settings) {
		// setup configuration
		settings = jQuery.extend({
			min: 0,
			defaultNumber: "",
			max: 0xFFFF,
			increment: 1,
			allowEmpty: true,
			allowFloat: false,
			upText: '&uarr;',
			downText: '&darr;'
		}, settings);
		
		if (settings.allowFloat)
			settings.regex = /^-?[0-9]+(\.[0-9]*)?$/;
		else
			settings.regex = /^-?[0-9]+$/;
	
		return this.each(setUpField);
		
		function setUpField() {
			if (this.tagName.toUpperCase() != 'INPUT')
				return;
			
			var jqSelf = jQuery(this);
			
			// Check values
			checkField(jqSelf);
			jqSelf.bind('change', function() { checkField(jQuery(this)) });
			
			removeIncrementors(jqSelf);
			addIncrementors(jqSelf);
		}
		
		function removeIncrementors(jqSelf) {
			var id = jqSelf.attr('id');
			jQuery('#nh_'+id).remove();
		}
		
		function addIncrementors(jqSelf) {
			var height = jqSelf.innerHeight();
			var id = jqSelf.attr('id');
						
			// Add incrementors
			var incrementors = jQuery('<div class="numberHelper" id="nh_'+id+'" style="display: -moz-inline-box; display: inline-block; position: relative; height: '+height+'px">'
				+'<span class="nhDown">'+settings.downText+'</span>'
				+'<span class="nhUp">'+settings.upText+'</span>'
				+'</div>');
			jqSelf.after(incrementors);
			
			// Register incrementors
			jQuery('.nhDown', incrementors).click(decrement);
			jQuery('.nhUp', incrementors).click(increment);
			
			placeIncrementors(id);
		}
		
		function placeIncrementors(id) {
			var theField = jQuery('#'+id);
			var theDiv = jQuery('div#nh_'+id);
			
			var top = parseInt(theField.css('border-top-width'))*2
				+ parseInt(theField.css('padding-top'))
				+ parseInt(theDiv.css('border-top-width'))*2;
			
			theDiv.css('top', top + 'px');
		}
		
		function checkField(jqField) {
			var val = jqField.val();
			matches = val.match(settings.regex);
			if (val == "" && settings.allowEmpty)
				;
			else if (settings.regex.test(val)) {
				if (val < settings.min)
					jqField.val(settings.min);
				else if (val > settings.max)
					jqField.val(settings.max);
			}
			else
				jqField.val(settings.defaultNumber);
		}
		
		function decrement(ev) {
			edit(jQuery(ev.target).parent().attr('id').substr(3), -1);
		}
		
		function increment(ev) {
			edit(jQuery(ev.target).parent().attr('id').substr(3), 1);
		}
		
		function edit(fieldID, factor) {
			var jqField = jQuery('#'+fieldID);
			var number = NaN;
			
			if (jqField.attr('disabled') == 'disabled' || jqField.attr('disabled') == true)
				return;
			
			if (settings.allowFloat)
				number = parseFloat(jqField.val());
			else
				number = parseInt(jqField.val());
			
			if (isNaN(number))
				number = settings.defaultNumber;
			else
				number = number + factor * settings.increment;
			
			jqField.val(number);
			jqField.trigger('change');
		}
	}
});

})();