/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
jQuery.fn.extend({
	tagField: function(acData, settings, acSettings) {
		var numFields = 0;
		// setup configuration
		settings = jQuery.extend({
			containerClass: 'tagsContainer',
			deleteText: 'Click to remove item',
			separator: ',',
			resetDelay: 50,
			blurDelay: 150
		}, settings);
		
		return this.each(function() {
			numFields++;
			jQuery.TagField(this, numFields++, acData, settings, acSettings);
		});
	}
});

jQuery.extend({
  TagField: function(input, holderNumber, acData, settings, acSettings) {
	var numItems = 0;
	var tagSet = new jQuery.TagField.Set();
	var container;
	var initialValue = jQuery(input).val();
	var jqInput = jQuery(input);
	
	// Add a class to input so that we can find it
	jqInput.addClass('TagFieldInput'+holderNumber);
	
	// Add an item
	var addItem = function(item) {
		if (! tagSet.add(item))
			return;
		
		numItems++;
		var id = 'TagFieldItem'+numItems;
		container.append('<span id="'+id+'" title="'+settings.deleteText+'">'+item+'</span> '); /* Preserve a space between span elements */
		jQuery('span#'+id).click(function() {
			tagSet.remove(jQuery(this).html());
			jQuery(this).remove();
		});
	}
	
	// Add a bunch of items
	var addAll = function(items) {
		items.forEach(addItem);
	}
	
	// Add all items from field, assuming they are separated by the given separator
	// The field is then cleared
	var loadFromInputAndClear = function() {
		value = jqInput.val();
		if (value.length > 0) {
			addAll(value.split(settings.separator));
			jqInput.val('');
		}
	}
	
	var clear = function() {
		tagSet.removeAll();
		container.html('');
	}
	
	var reset_real = function() {
		clear();
		jqInput.val(initialValue);
		loadFromInputAndClear();
	}
	
	var reset = function(resetDelay) {
		window.setTimeout(reset_real, settings.resetDelay);
	}
	
	// Register blur to parse items when the field is left. This way we can add
	// tags not in the completion list
	jqInput.blur(function() {
		// Delay so that the autocompleter has time to clear the field. Else,
		// for "fr" matching "freedom", both "fr" and "freedom" would be added
		window.setTimeout(loadFromInputAndClear, settings.blurDelay);
	});
	
	// Register form submission
	jQuery(input.form).submit(function() {
		// Add currently entered but not validated tags
		loadFromInputAndClear();
		// Restore a CSV list
		jqInput.val(tagSet.toArray().join(','));
	});
	
	// Register form reset
	jQuery(input.form).attr('onreset', 'jQuery("input.TagFieldInput'+holderNumber+'").trigger("TagField.reset")');
	
	// Some events
	jqInput.bind('TagField.add', function() { addItem(arguments[1]); });
	jqInput.bind('TagField.clear', function() { clear(); });
	jqInput.bind('TagField.reset', function() { reset(); });
	
	
	
	// Create container
	jqInput.after('<div id="TagFieldContainer'+holderNumber+'" class="'+settings.containerClass+'"></div>');
	container = jQuery('div#TagFieldContainer'+holderNumber);
	
	// Enable autocompletion
	jqInput.autocomplete(acData, acSettings).result(function(event, item) {
		jqInput.val('');
		jqInput.trigger('TagField.add', item);
	});
	
	// If the field is not empty, load its values
	loadFromInputAndClear();
}});





jQuery.TagField.Set = function() {
	this.backend = Array();
};

/** Returns true if something was added, false if the item was already present */
jQuery.TagField.Set.prototype.add = function(item) {
	if (this.backend.indexOf(item) != -1)
		return false;
	
	this.backend.push(item);
	return true;
};

jQuery.TagField.Set.prototype.remove = function(item) {
	var index;
	if ((index = this.backend.indexOf(item)) == -1)
		return;
	
	this.backend.splice(index, 1);
};

jQuery.TagField.Set.prototype.removeAll = function() {
	this.backend = Array();
};

jQuery.TagField.Set.prototype.toArray = function() {
	return this.backend.slice();
};
