#! /bin/sh
INSTALL_OS=''
INSTALL_MANAGER=''
APACHE_GROUP=''
APACHE_USER=''
APACHE_DOCROOT=''

# Has a side effect of setting INSTALL_OS, INSTALL_MANAGER, APACHE_USER, APACHE_GROUP, APACHE_DOCROOT
detect_os_and_method() {
	if grep Debian /etc/issue > /dev/null 2>&1; then
		INSTALL_OS='Debian'
		INSTALL_MANAGER='aptitude'
		APACHE_USER='www-data'
		APACHE_GROUP='www-data'
		APACHE_DOCROOT='/var/www'
	elif [ -r /etc/fedora-release ]; then
		INSTALL_OS='Fedora'
		INSTALL_MANAGER='yum'
		APACHE_USER='apache'
		APACHE_GROUP='apache'
		APACHE_DOCROOT='/var/www/html'
	elif [ -r /etc/mandriva-release ]; then
		INSTALL_OS='Mandriva'
		INSTALL_MANAGER='urpmi'
		APACHE_USER='apache'
		APACHE_GROUP='apache'
		APACHE_DOCROOT='/var/www/html'
	elif grep openSUSE /etc/issue > /dev/null 2>&1; then
		INSTALL_OS='openSUSE'
		INSTALL_MANAGER='zypper'
		APACHE_USER='wwwrun'
		APACHE_GROUP='www'
		APACHE_DOCROOT='/srv/www/htdocs'
	elif [ -r /etc/lsb-release ]; then
		. /etc/lsb-release
		if [ "$DISTRIB_ID" = "Ubuntu" ]; then
			INSTALL_OS='Ubuntu'
			INSTALL_MANAGER='aptitude'
			APACHE_USER='www-data'
			APACHE_GROUP='www-data'
			APACHE_DOCROOT='/var/www'
		fi
	fi

	if [ -n "$INSTALL_OS" ]; then
		echo "I detected $INSTALL_OS and will be using $INSTALL_MANAGER for installation if need be."
		echo "You can change this recommended configuration if you know what you are doing."
		echo "Do you? y/[n]"
		read choice
		if ([ -z "$choice" ] || [ "$choice" = "n" ]); then
			true
		elif [ "$choice" = "y" ]; then
			manual_os_and_method
		else
			echo "Not a valid choice"
			exit 1
		fi
	else
		echo "I failed to detect your OS. You will have to choose by yourself"
		manual_os_and_method
	fi

	echo "Using $INSTALL_OS with $INSTALL_MANAGER..."
}

manual_os_and_method() {
	echo "Choose your method:"
	printf "\t1. Debian (or Debian-based) + aptitude\n"
	printf "\t2. Fedora (or Fedora-based) + yum\n"
	printf "\t3. Mandriva (or Fedora-based) + yum\n"
	printf "\t4. openSUSE (or SUSE-based) + zypper\n"
	printf "\t5. Ubuntu (or Ubuntu-based) + aptitude\n"
	printf "\tC. Cancel installation\n"
	echo "Your choice [C]: "
	read choice
	case $choice in
		"1")
		INSTALL_OS='Debian'
		INSTALL_MANAGER='aptitude'
		APACHE_USER='www-data'
		APACHE_GROUP='www-data'
		APACHE_DOCROOT='/var/www'
		;;

		"2")
		INSTALL_OS='Fedora'
		INSTALL_MANAGER='yum'
		APACHE_USER='apache'
		APACHE_GROUP='apache'
		APACHE_DOCROOT='/var/www/html'
		;;

		"3")
		INSTALL_OS='Mandriva'
		INSTALL_MANAGER='urpmi'
		APACHE_USER='apache'
		APACHE_GROUP='apache'
		APACHE_DOCROOT='/var/www/html'
		;;

		"4")
		INSTALL_OS='openSUSE'
		INSTALL_MANAGER='zypper'
		APACHE_USER='wwwrun'
		APACHE_GROUP='www'
		APACHE_DOCROOT='/srv/www/htdocs'
		;;

		"5")
		INSTALL_OS='Ubuntu'
		INSTALL_MANAGER='aptitude'
		APACHE_USER='www-data'
		APACHE_GROUP='www-data'
		APACHE_DOCROOT='/var/www'
		;;

		"C")
		echo "Aborting installation"
		exit 2
		;;

		*)
		echo "Not a valid choice"
		exit 1
	esac
}

patch_php_ini() {
	ini=$1
	
	echo "Should I patch the PHP configuration to increase limits"
	echo "(highly recommended to avoid import/export problems)? [y]/n"
	read choice
	if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
		sed -i 's/^max_execution_time[[:space:]]*=[[:space:]]*[0-9]\+/max_execution_time = 600/' $ini
		sed -i 's/^max_input_time[[:space:]]*=[[:space:]]*[0-9]\+/max_input_time = 600/' $ini
		sed -i 's/^memory_limit[[:space:]]*=[[:space:]]*[0-9]\+M/memory_limit = 64M/' $ini
		sed -i 's/^post_max_size[[:space:]]*=[[:space:]]*[0-9]\+M/post_max_size = 256M/' $ini
		sed -i 's/^upload_max_filesize[[:space:]]*=[[:space:]]*[0-9]\+M/upload_max_filesize = 256M/' $ini
	elif [ "$choice" = "n" ]; then
		true # do nothing
	else
		echo "Not a valid choice"
		exit 1
	fi
}

restrict_apache_local() {
	config=$1

	echo "Should I restrict the Apache access to the local computer? [y]/n"
	read choice
	if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
		sed -i 's/^\(\s*\)Listen \([0-9]\+\)$/\1Listen 127.0.0.1:\2/' $config
	elif [ "$choice" = "n" ]; then
		true # do nothing
	else
		echo "Not a valid choice"
		exit 1
	fi
}

add_demyo_apache_config() { # BASE_PATH CONFIG_FILE
	echo "<Directory $1/demyo>
	AllowOverride FileInfo Indexes
	Options +FollowSymLinks
</Directory>" > $2
}

mysql_legacy_secure() { # SERVICE_NAME
	echo "Should I secure the MySQL server? [y]/n"
	read choice
	if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
		echo "You will be asked for a new root password for MySQL. Remember it."
		sleep 2
		service $1 start
		mysql_secure_installation
		sed -i 's/\[mysqld\]/[mysqld]\nbind-address = 127.0.0.1/' /etc/my.cnf
	elif [ "$choice" = "n" ]; then
		true # do nothing
	else
		echo "Not a valid choice"
		exit 1
	fi
}

