#! /bin/sh
# Copyright 2007-2009 Xavier 'Xr' Dalem
# 
# This file is part of Demyo. Demyo is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
# 
# Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#		Debian
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

install_Debian() {
	echo "If asked wether the SQL server should be available from the network, answer no."
	echo "You will be asked for a root password for MySQL. Remember it."
	sleep 2
	aptitude install -R apache2 libapache2-mod-php5 mysql-server php5-mysql php5-gd php-apc
}

configure_Debian() {
	a2enmod deflate
	a2enmod expires
	a2enmod rewrite
	
	add_demyo_apache_config $1 /etc/apache2/sites-available/demyo
	ln -s /etc/apache2/sites-available/demyo /etc/apache2/sites-enabled/demyo
	
	restrict_apache_local '/etc/apache2/ports.conf'
	patch_php_ini '/etc/php5/apache2/php.ini'
	
	invoke-rc.d apache2 restart
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#		Fedora Core
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

install_Fedora() {
	yum install httpd php mysql mysql-server php-mysql php-gd php-pecl-apc
}

configure_Fedora() {
	add_demyo_apache_config $1 /etc/httpd/conf.d/demyo.conf

	restrict_apache_local '/etc/httpd/conf/httpd.conf'
	patch_php_ini '/etc/php.ini'

	mysql_legacy_secure mysqld

	chkconfig httpd on
	chkconfig mysqld on
	service httpd restart
	service mysqld restart
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#		Mandriva
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

install_Mandriva() {
	urpmi apache-base apache-mpm-prefork apache-mod_deflate apache-mod_php mysql mysql-client php-mysql php-gd php-apc
}

configure_Mandriva() {
	configure_Fedora $@
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#		openSUSE
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

install_openSUSE() {
	PACKAGES='apache2 apache2-mod_php5 mysql mysql-client php5-mysql php5-gd'
	
	if zypper repos -u | grep 'server:/php:/extensions/' > /dev/null 2>&1; then
		PACKAGES="$PACKAGES php5-APC"
	else
		echo "I need an additionnal repository provided by SUSE for optimal performance."
		echo "You can either let me add it, or proceed and do without the extra packages."

		echo "Add the repository? [y]/n"
		read choice
		if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
			OPENSUSE_VERSION=$(sed 's/.*SUSE \(..\..\).*/\1/' /etc/issue)
			zypper ar http://download.opensuse.org/repositories/server:/php:/extensions/openSUSE_$OPENSUSE_VERSION/ openSUSE-$OPENSUSE_VERSION-PHP-Extensions
			zypper refresh
			PACKAGES="$PACKAGES php5-APC"
		elif [ "$choice" = "n" ]; then
			true # do nothing
		else
			echo "Not a valid choice"
			exit 1
		fi
	fi

	zypper install $PACKAGES
}

configure_openSUSE() {
	sysconf_addword /etc/sysconfig/apache2 APACHE_MODULES deflate
	sysconf_addword /etc/sysconfig/apache2 APACHE_MODULES expires
	sysconf_addword /etc/sysconfig/apache2 APACHE_MODULES rewrite

	add_demyo_apache_config $1 /etc/apache2/conf.d/demyo.conf

	restrict_apache_local '/etc/apache2/listen.conf'
	patch_php_ini '/etc/php5/apache2/php.ini'

	mysql_legacy_secure mysql

	chkconfig apache2 on
	chkconfig mysql on
	service apache2 restart
	service mysql restart
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#		Ubuntu
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

install_Ubuntu() {
	if ! grep '^[^#].*universe' /etc/apt/sources.list >/dev/null; then
		echo "You should enable the universe repository to benefit from maximum performance."
		echo "You can either abort the installation, add the repository and restart the"
		echo "installation; or proceed and do without the extra packages."

		echo "Abort? [y]/n"
		read choice
		if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
			exit 2
		elif [ "$choice" = "n" ]; then
			true # do nothing
		else
			echo "Not a valid choice"
			exit 1
		fi
	fi

	install_Debian $@
}
configure_Ubuntu() {
	configure_Debian $@
}
