#! /bin/sh
# Copyright 2007-2009 Xavier 'Xr' Dalem
# 
# This file is part of Demyo. Demyo is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
# 
# Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.

set -e
if [ `id -u` -ne 0 ] ; then
	echo "This script requires superuser privileges."
	exit 1
fi

export PATH=/sbin:/usr/sbin:$PATH
cd `dirname $0`
. ./functions-common.sh
. ./functions-os.sh


printf "Demyo installer.\n\nYou will be asked for a set of questions.
Most have decent default answers (just press Enter) but they can give you more
control over your installation if you really need it.\n\n"
sleep 2


detect_os_and_method



# Easier to ask it now
echo "Where should I install Demyo? [$APACHE_DOCROOT]"
read choice
if [ -z "$choice" ]; then
	INSTALL_TO=$APACHE_DOCROOT
else
	INSTALL_TO=$choice
fi

# Check if the directory exists
if [ -e $INSTALL_TO/demyo ]; then
	echo "$INSTALL_TO/demyo Already exists. If you already installed demyo,
you need to update instead."
	exit 1
fi



# Install requirements
echo "Demyo requires Apache2, MySQL, PHP5 and some PHP modules.
What should I do now ?
1. Proceed. I'm sure these are installed.
2. Stop. I'll install them manually.
3. Try to install them automatically with $INSTALL_MANAGER."
echo "Your choice? [3] "
read choice

if [ "$choice" = "1" ]; then
	echo "Assuming requirements are installed"
elif [ "$choice" = "2" ]; then
	echo "Aborting installation"
	exit 2
elif ([ -z "$choice" ] || [ "$choice" = "3" ]); then
	echo "Installing automatically"
	eval install_$INSTALL_OS
else
	echo "Not a valid choice"
	exit 1
fi



# Configure
echo
echo "Specific configuration is needed to run Demyo.
What should be done now ?
1. Proceed. I'm sure the packages are well configured.
2. Stop. I'll configure them manually.
3. Try to configure them automatically."
echo "Your choice? [3] "
read choice

if [ "$choice" = "1" ]; then
	echo "Assuming everything is configured"
elif [ "$choice" = "2" ]; then
	echo "Aborting installation"
	exit 2
elif ([ -z "$choice" ] || [ "$choice" = "3" ]); then
	echo "Trying to configure automatically."
	eval configure_$INSTALL_OS $INSTALL_TO
else
	echo "Not a valid choice"
	exit 1
fi



# Install software
echo
mkdir -p $INSTALL_TO
cp -r demyo $INSTALL_TO/demyo

# For who ?
DEMYO_USER=`stat -c %U demyo`
echo "Install demyo for which user? [$DEMYO_USER]"
read choice
if [ -n "$choice" ]; then
	DEMYO_USER=$choice
fi

# Change rights
chown -R $DEMYO_USER:$APACHE_GROUP $INSTALL_TO/demyo
chmod -R u=rwX,g=rX,o= $INSTALL_TO/demyo
chmod -R g+w $INSTALL_TO/demyo/app/tmp $INSTALL_TO/demyo/app/webroot/img/thumbs
chmod g+w $INSTALL_TO/demyo/app/config $INSTALL_TO/demyo/app/config/demyo.php
chown -R $APACHE_USER $INSTALL_TO/demyo/app/webroot/img/collection_images
chmod -R go=rwX $INSTALL_TO/demyo/app/webroot/img/collection_images
chmod -R g=rwX $INSTALL_TO/demyo/app/webroot/ccss $INSTALL_TO/demyo/app/webroot/cjs




# Install the SQL database
echo "Should I create a MySQL database for demyo ? [y]/n"
read choice
DB_NAME=demyo
if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
	echo "Name of the database ? [demyo]"
	read choice
	if [ -n "$choice" ]; then
		DB_NAME=$choice
	fi
	
	echo "You will be prompted for the root password for the MySQL server."
	echo 'CREATE DATABASE `'$DB_NAME'` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;' | mysql -u root -p
elif [ "$choice" = "n" ]; then
	true # do nothing
else
	echo "Not a valid choice"
	exit 1
fi




# Install the SQL user
echo "Should I create a MySQL user for demyo ? [y]/n"
read choice
if ([ -z "$choice" ] || [ "$choice" = "y" ]); then
	echo "Name of the user ? [demyo]"
	read choice
	if [ -z "$choice" ]; then
		DB_USER=demyo
	else
		DB_USER=$choice
	fi
	
	echo "Password of the user ? [password]"
	read choice
	if [ -z "$choice" ]; then
		DB_PASS=password
	else
		DB_PASS=$choice
	fi
	
	echo "You will be prompted for the root password for the MySQL server."
	echo "CREATE USER '$DB_USER'@'localhost' IDENTIFIED BY '$DB_PASS';
	GRANT SELECT , INSERT , UPDATE , DELETE , CREATE , DROP , INDEX , ALTER , LOCK TABLES ON ${DB_NAME}. * TO '$DB_USER'@'localhost' IDENTIFIED BY '$DB_PASS' WITH MAX_QUERIES_PER_HOUR 0 MAX_CONNECTIONS_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 MAX_USER_CONNECTIONS 0 ;" | mysql -u root -p
elif [ "$choice" = "n" ]; then
	true # do nothing
else
	echo "Not a valid choice"
	exit 1
fi




echo "Demyo is now installed. Visit its home page (default: http://localhost/demyo/ )
to fill the last information (some of which you got here)."
